# Copyright (c) 2016 Juniper Networks Inc.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
from unittest import mock

from osc_lib.utils import columns as column_util

from neutronclient.osc import utils as nc_osc_utils
from neutronclient.osc.v2.networking_bgpvpn import constants
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    CreateBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    DeleteBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    ListBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    SetBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    ShowBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    UnsetBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.router_association import\
    CreateBgpvpnRouterAssoc
from neutronclient.osc.v2.networking_bgpvpn.router_association import\
    SetBgpvpnRouterAssoc
from neutronclient.osc.v2.networking_bgpvpn.router_association import\
    ShowBgpvpnRouterAssoc
from neutronclient.tests.unit.osc.v2 import fakes as test_fakes


_FAKE_PROJECT_ID = 'fake_project_id'


class TestNeutronClientBgpvpn(test_fakes.TestNeutronClientOSCV2):

    def setUp(self):
        super(TestNeutronClientBgpvpn, self).setUp()
        self.neutronclient.find_resource = mock.Mock(
            side_effect=lambda resource, name_or_id, project_id=None,
            cmd_resource=None, parent_id=None, fields=None:
            {'id': name_or_id, 'tenant_id': _FAKE_PROJECT_ID})
        self.neutronclient.find_resource_by_id = mock.Mock(
            side_effect=lambda resource, resource_id, cmd_resource=None,
            parent_id=None, fields=None:
            {'id': resource_id, 'tenant_id': _FAKE_PROJECT_ID})
        nc_osc_utils.find_project = mock.Mock(
            side_effect=lambda _, name_or_id, __: mock.Mock(id=name_or_id))


class FakeBgpvpn(object):
    """Fake BGP VPN with attributes."""

    @staticmethod
    def create_one_bgpvpn(attrs=None):
        """Create a fake BGP VPN."""

        attrs = attrs or {}

        # Set default attributes.
        bgpvpn_attrs = {
            'id': 'fake_bgpvpn_id',
            'tenant_id': _FAKE_PROJECT_ID,
            'name': '',
            'type': 'l3',
            'route_targets': [],
            'import_targets': [],
            'export_targets': [],
            'route_distinguishers': [],
            'networks': [],
            'routers': [],
            'ports': [],
            'vni': 100,
            'local_pref': 777,
        }

        # Overwrite default attributes.
        bgpvpn_attrs.update(attrs)

        return copy.deepcopy(bgpvpn_attrs)

    @staticmethod
    def create_bgpvpns(attrs=None, count=1):
        """Create multiple fake BGP VPN."""

        bgpvpns = []
        for i in range(0, count):
            if attrs is None:
                attrs = {'id': 'fake_id%d' % i}
            elif getattr(attrs, 'id', None) is None:
                attrs['id'] = 'fake_id%d' % i
            bgpvpns.append(FakeBgpvpn.create_one_bgpvpn(attrs))

        return {constants.BGPVPNS: bgpvpns}


class BgpvpnFakeAssoc(object):
    _assoc_res_name = 'fake_resource'
    _resource = '%s_association' % _assoc_res_name
    _resource_plural = '%ss' % _resource

    _attr_map = (
        ('id', 'ID', column_util.LIST_BOTH),
        ('tenant_id', 'Project', column_util.LIST_LONG_ONLY),
        ('%s_id' % _assoc_res_name, '%s ID' % _assoc_res_name.capitalize(),
         column_util.LIST_BOTH),
    )
    _formatters = {}


class CreateBgpvpnFakeResAssoc(BgpvpnFakeAssoc, CreateBgpvpnResAssoc):
    pass


class SetBgpvpnFakeResAssoc(BgpvpnFakeAssoc, SetBgpvpnResAssoc):
    pass


class UnsetBgpvpnFakeResAssoc(BgpvpnFakeAssoc, UnsetBgpvpnResAssoc):
    pass


class DeleteBgpvpnFakeResAssoc(BgpvpnFakeAssoc, DeleteBgpvpnResAssoc):
    pass


class ListBgpvpnFakeResAssoc(BgpvpnFakeAssoc, ListBgpvpnResAssoc):
    pass


class ShowBgpvpnFakeResAssoc(BgpvpnFakeAssoc, ShowBgpvpnResAssoc):
    pass


class BgpvpnFakeRouterAssoc(object):
    _assoc_res_name = 'fake_resource'
    _resource = '%s_association' % _assoc_res_name
    _resource_plural = '%ss' % _resource

    _attr_map = (
        ('id', 'ID', column_util.LIST_BOTH),
        ('tenant_id', 'Project', column_util.LIST_LONG_ONLY),
        ('%s_id' % _assoc_res_name, '%s ID' % _assoc_res_name.capitalize(),
         column_util.LIST_BOTH),
        ('advertise_extra_routes', 'Advertise extra routes',
         column_util.LIST_LONG_ONLY),
    )
    _formatters = {}


class CreateBgpvpnFakeRouterAssoc(BgpvpnFakeRouterAssoc,
                                  CreateBgpvpnRouterAssoc):
    pass


class SetBgpvpnFakeRouterAssoc(BgpvpnFakeRouterAssoc, SetBgpvpnRouterAssoc):
    pass


class ShowBgpvpnFakeRouterAssoc(BgpvpnFakeRouterAssoc, ShowBgpvpnRouterAssoc):
    pass


class FakeResource(object):
    """Fake resource with minimal attributes."""

    @staticmethod
    def create_one_resource(attrs=None):
        """Create a fake resource."""

        attrs = attrs or {}

        # Set default attributes.
        res_attrs = {
            'id': 'fake_resource_id',
            'tenant_id': _FAKE_PROJECT_ID,
        }

        # Overwrite default attributes.
        res_attrs.update(attrs)
        return copy.deepcopy(res_attrs)

    @staticmethod
    def create_resources(attrs=None, count=1):
        """Create multiple fake resources."""

        resources = []
        for i in range(0, count):
            if attrs is None:
                attrs = {'id': 'fake_id%d' % i}
            elif getattr(attrs, 'id', None) is None:
                attrs['id'] = 'fake_id%d' % i
            resources.append(FakeResource.create_one_resource(attrs))

        return {'%ss' % BgpvpnFakeAssoc._assoc_res_name: resources}


class FakeResAssoc(object):
    """Fake resource association with minimal attributes."""

    @staticmethod
    def create_one_resource_association(resource, attrs=None):
        """Create a fake resource association."""

        attrs = attrs or {}

        res_assoc_attrs = {
            'id': 'fake_association_id',
            'tenant_id': resource['tenant_id'],
            'fake_resource_id': resource['id'],
        }

        # Overwrite default attributes.
        res_assoc_attrs.update(attrs)
        return copy.deepcopy(res_assoc_attrs)

    @staticmethod
    def create_resource_associations(resources):
        """Create multiple fake resource associations."""

        res_assocs = []
        for idx, resource in enumerate(
                resources['%ss' % BgpvpnFakeAssoc._assoc_res_name]):
            res_assoc_attrs = {
                'id': 'fake_association_id%d' % idx,
                'tenant_id': resource['tenant_id'],
                'fake_resource_id': resource['id'],
            }
            res_assocs.append(copy.deepcopy(res_assoc_attrs))

        return {BgpvpnFakeAssoc._resource_plural: res_assocs}



# Copyright 2013 Mirantis Inc.
# Copyright 2014 Blue Box Group, Inc.
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


def _get_pool_id(client, pool_id_or_name):
    return neutronV20.find_resourceid_by_name_or_id(client, 'pool',
                                                    pool_id_or_name,
                                                    cmd_resource='lbaas_pool')


class LbaasMemberMixin(object):

    def set_extra_attrs(self, parsed_args):
        self.parent_id = _get_pool_id(self.get_client(), parsed_args.pool)

    def add_known_arguments(self, parser):
        parser.add_argument(
            'pool', metavar='POOL',
            help=_('ID or name of the pool that this member belongs to.'))


def _add_common_args(parser):
    parser.add_argument(
        '--name',
        help=_('Name of the member.'))
    parser.add_argument(
        '--weight',
        help=_('Weight of the member in the pool (default:1, [0..256]).'))


def _parse_common_args(body, parsed_args):
    neutronV20.update_dict(parsed_args, body, ['weight', 'name'])


class ListMember(LbaasMemberMixin, neutronV20.ListCommand):
    """LBaaS v2 List members that belong to a given pool."""

    resource = 'member'
    shadow_resource = 'lbaas_member'
    list_columns = [
        'id', 'name', 'address', 'protocol_port', 'weight',
        'subnet_id', 'admin_state_up', 'status'
    ]
    pagination_support = True
    sorting_support = True

    def take_action(self, parsed_args):
        self.parent_id = _get_pool_id(self.get_client(), parsed_args.pool)
        self.values_specs.append('--pool_id=%s' % self.parent_id)
        return super(ListMember, self).take_action(parsed_args)


class ShowMember(LbaasMemberMixin, neutronV20.ShowCommand):
    """LBaaS v2 Show information of a given member."""

    resource = 'member'
    shadow_resource = 'lbaas_member'


class CreateMember(neutronV20.CreateCommand):
    """LBaaS v2 Create a member."""

    resource = 'member'
    shadow_resource = 'lbaas_member'

    def add_known_arguments(self, parser):
        _add_common_args(parser)
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state', action='store_false',
            help=_('Set admin state up to false.'))
        parser.add_argument(
            '--subnet',
            required=True,
            help=_('Subnet ID or name for the member.'))
        parser.add_argument(
            '--address',
            required=True,
            help=_('IP address of the pool member in the pool.'))
        parser.add_argument(
            '--protocol-port',
            required=True,
            help=_('Port on which the pool member listens for requests or '
                   'connections.'))
        parser.add_argument(
            'pool', metavar='POOL',
            help=_('ID or name of the pool that this member belongs to.'))

    def args2body(self, parsed_args):
        self.parent_id = _get_pool_id(self.get_client(), parsed_args.pool)
        _subnet_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'subnet', parsed_args.subnet)
        body = {'subnet_id': _subnet_id,
                'admin_state_up': parsed_args.admin_state,
                'protocol_port': parsed_args.protocol_port,
                'address': parsed_args.address}
        neutronV20.update_dict(parsed_args, body,
                               ['subnet_id', 'tenant_id'])
        _parse_common_args(body, parsed_args)
        return {self.resource: body}


class UpdateMember(neutronV20.UpdateCommand):
    """LBaaS v2 Update a given member."""

    resource = 'member'
    shadow_resource = 'lbaas_member'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'pool', metavar='POOL',
            help=_('ID or name of the pool that this member belongs to.'))
        utils.add_boolean_argument(
            parser, '--admin-state-up',
            help=_('Update the administrative state of '
                   'the member (True meaning "Up").'))
        _add_common_args(parser)

    def args2body(self, parsed_args):
        self.parent_id = _get_pool_id(self.get_client(), parsed_args.pool)
        body = {}
        if hasattr(parsed_args, "admin_state_up"):
            body['admin_state_up'] = parsed_args.admin_state_up
        _parse_common_args(body, parsed_args)
        return {self.resource: body}


class DeleteMember(LbaasMemberMixin, neutronV20.DeleteCommand):
    """LBaaS v2 Delete a given member."""

    resource = 'member'
    shadow_resource = 'lbaas_member'



# Copyright 2013 Big Switch Networks
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import argparse

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronv20


def _add_common_args(parser, is_create=True):
    """If is_create is True, protocol and action become mandatory arguments.

    CreateCommand = is_create : True
    UpdateCommand = is_create : False
    """
    parser.add_argument(
        '--name',
        help=_('Name for the firewall rule.'))
    parser.add_argument(
        '--description',
        help=_('Description for the firewall rule.'))
    parser.add_argument(
        '--source-ip-address',
        help=_('Source IP address or subnet.'))
    parser.add_argument(
        '--destination-ip-address',
        help=_('Destination IP address or subnet.'))
    parser.add_argument(
        '--source-port',
        help=_('Source port (integer in [1, 65535] or range in a:b).'))
    parser.add_argument(
        '--destination-port',
        help=_('Destination port (integer in [1, 65535] or range in '
               'a:b).'))
    utils.add_boolean_argument(
        parser, '--enabled', dest='enabled',
        help=_('Whether to enable or disable this rule.'))
    parser.add_argument(
        '--protocol', choices=['tcp', 'udp', 'icmp', 'any'],
        required=is_create,
        type=utils.convert_to_lowercase,
        help=_('Protocol for the firewall rule.'))
    parser.add_argument(
        '--action',
        required=is_create,
        type=utils.convert_to_lowercase,
        choices=['allow', 'deny', 'reject'],
        help=_('Action for the firewall rule.'))


def common_args2body(parsed_args):
    body = {}
    neutronv20.update_dict(parsed_args, body,
                           ['name', 'description', 'shared', 'tenant_id',
                            'source_ip_address', 'destination_ip_address',
                            'source_port', 'destination_port', 'action',
                            'enabled', 'ip_version'])
    protocol = parsed_args.protocol
    if protocol:
        if protocol == 'any':
            protocol = None
        body['protocol'] = protocol
    return body


class ListFirewallRule(neutronv20.ListCommand):
    """List firewall rules that belong to a given tenant."""

    resource = 'firewall_rule'
    list_columns = ['id', 'name', 'firewall_policy_id', 'summary', 'enabled']
    pagination_support = True
    sorting_support = True

    def extend_list(self, data, parsed_args):
        for d in data:
            val = []
            if d.get('protocol'):
                protocol = d['protocol'].upper()
            else:
                protocol = 'no-protocol'
            val.append(protocol)
            if 'source_ip_address' in d and 'source_port' in d:
                src = 'source: ' + str(d['source_ip_address']).lower()
                src = src + '(' + str(d['source_port']).lower() + ')'
            else:
                src = 'source: none specified'
            val.append(src)
            if 'destination_ip_address' in d and 'destination_port' in d:
                dst = 'dest: ' + str(d['destination_ip_address']).lower()
                dst = dst + '(' + str(d['destination_port']).lower() + ')'
            else:
                dst = 'dest: none specified'
            val.append(dst)
            if 'action' in d:
                action = d['action']
            else:
                action = 'no-action'
            val.append(action)
            d['summary'] = ',\n '.join(val)


class ShowFirewallRule(neutronv20.ShowCommand):
    """Show information of a given firewall rule."""

    resource = 'firewall_rule'


class CreateFirewallRule(neutronv20.CreateCommand):
    """Create a firewall rule."""

    resource = 'firewall_rule'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--shared',
            action='store_true',
            help=_('Set shared flag for the firewall rule.'),
            default=argparse.SUPPRESS)
        _add_common_args(parser)
        parser.add_argument(
            '--ip-version',
            type=int, choices=[4, 6], default=4,
            help=_('IP version for the firewall rule (default is 4).'))

    def args2body(self, parsed_args):
        return {self.resource: common_args2body(parsed_args)}


class UpdateFirewallRule(neutronv20.UpdateCommand):
    """Update a given firewall rule."""

    resource = 'firewall_rule'

    def add_known_arguments(self, parser):
        utils.add_boolean_argument(
            parser,
            '--shared',
            dest='shared',
            help=_('Update the shared flag for the firewall rule.'),
            default=argparse.SUPPRESS)
        parser.add_argument(
            '--ip-version',
            type=int, choices=[4, 6],
            help=_('Update IP version for the firewall rule.'))
        _add_common_args(parser, is_create=False)

    def args2body(self, parsed_args):
        return {self.resource: common_args2body(parsed_args)}


class DeleteFirewallRule(neutronv20.DeleteCommand):
    """Delete a given firewall rule."""

    resource = 'firewall_rule'



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from unittest import mock

from neutronclient.osc.v2.dynamic_routing import bgp_speaker
from neutronclient.tests.unit.osc.v2.dynamic_routing import fakes


class TestListBgpSpeaker(fakes.TestNeutronDynamicRoutingOSCV2):
    _bgp_speakers = fakes.FakeBgpSpeaker.create_bgp_speakers()
    columns = ('ID', 'Name', 'Local AS', 'IP Version')
    data = []
    for _bgp_speaker in _bgp_speakers['bgp_speakers']:
        data.append((
            _bgp_speaker['id'],
            _bgp_speaker['name'],
            _bgp_speaker['local_as'],
            _bgp_speaker['ip_version']))

    def setUp(self):
        super(TestListBgpSpeaker, self).setUp()

        self.neutronclient.list_bgp_speakers = mock.Mock(
            return_value=self._bgp_speakers
        )

        # Get the command object to test
        self.cmd = bgp_speaker.ListBgpSpeaker(self.app, self.namespace)

    def test_bgp_speaker_list(self):
        parsed_args = self.check_parser(self.cmd, [], [])

        columns, data = self.cmd.take_action(parsed_args)
        self.neutronclient.list_bgp_speakers.assert_called_once_with()
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestDeleteBgpSpeaker(fakes.TestNeutronDynamicRoutingOSCV2):

    _bgp_speaker = fakes.FakeBgpSpeaker.create_one_bgp_speaker()

    def setUp(self):
        super(TestDeleteBgpSpeaker, self).setUp()

        self.neutronclient.delete_bgp_speaker = mock.Mock(return_value=None)

        self.cmd = bgp_speaker.DeleteBgpSpeaker(self.app, self.namespace)

    def test_delete_bgp_speaker(self):
        arglist = [
            self._bgp_speaker['name'],
        ]
        verifylist = [
            ('bgp_speaker', self._bgp_speaker['name']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.neutronclient.delete_bgp_speaker.assert_called_once_with(
            self._bgp_speaker['name'])
        self.assertIsNone(result)


class TestShowBgpSpeaker(fakes.TestNeutronDynamicRoutingOSCV2):
    _one_bgp_speaker = fakes.FakeBgpSpeaker.create_one_bgp_speaker()
    data = (
        _one_bgp_speaker['advertise_floating_ip_host_routes'],
        _one_bgp_speaker['advertise_tenant_networks'],
        _one_bgp_speaker['id'],
        _one_bgp_speaker['ip_version'],
        _one_bgp_speaker['local_as'],
        _one_bgp_speaker['name'],
        _one_bgp_speaker['networks'],
        _one_bgp_speaker['peers'],
        _one_bgp_speaker['tenant_id']
    )
    _bgp_speaker = {'bgp_speaker': _one_bgp_speaker}
    _bgp_speaker_name = _one_bgp_speaker['name']
    columns = (
        'advertise_floating_ip_host_routes',
        'advertise_tenant_networks',
        'id',
        'ip_version',
        'local_as',
        'name',
        'networks',
        'peers',
        'tenant_id'
    )

    def setUp(self):
        super(TestShowBgpSpeaker, self).setUp()

        self.neutronclient.show_bgp_speaker = mock.Mock(
            return_value=self._bgp_speaker
        )
        # Get the command object to test
        self.cmd = bgp_speaker.ShowBgpSpeaker(self.app, self.namespace)

    def test_bgp_speaker_show(self):
        arglist = [
            self._bgp_speaker_name,
        ]
        verifylist = [
            ('bgp_speaker', self._bgp_speaker_name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        data = self.cmd.take_action(parsed_args)
        self.neutronclient.show_bgp_speaker.assert_called_once_with(
            self._bgp_speaker_name)
        self.assertEqual(self.columns, data[0])
        self.assertEqual(self.data, data[1])


class TestSetBgpSpeaker(fakes.TestNeutronDynamicRoutingOSCV2):
    _one_bgp_speaker = fakes.FakeBgpSpeaker.create_one_bgp_speaker()
    _bgp_speaker_name = _one_bgp_speaker['name']

    def setUp(self):
        super(TestSetBgpSpeaker, self).setUp()
        self.neutronclient.update_bgp_speaker = mock.Mock(
            return_value=None)

        self.cmd = bgp_speaker.SetBgpSpeaker(self.app, self.namespace)

    def test_set_bgp_speaker(self):
        arglist = [
            self._bgp_speaker_name,
            '--name', 'noob',
        ]
        verifylist = [
            ('bgp_speaker', self._bgp_speaker_name),
            ('name', 'noob'),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        attrs = {'bgp_speaker': {
            'name': 'noob'}
        }
        self.neutronclient.update_bgp_speaker.assert_called_once_with(
            self._bgp_speaker_name, attrs)
        self.assertIsNone(result)



# Copyright 2015 Huawei Technologies India Pvt Ltd, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#


from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.neutron import v2_0 as neutronv20
from neutronclient.neutron.v2_0.qos import rule as qos_rule


BANDWIDTH_LIMIT_RULE_RESOURCE = 'bandwidth_limit_rule'


def add_bandwidth_limit_arguments(parser):
    parser.add_argument(
        '--max-kbps',
        help=_('Maximum bandwidth in kbps.'))
    parser.add_argument(
        '--max-burst-kbps',
        help=_('Maximum burst bandwidth in kbps.'))


def update_bandwidth_limit_args2body(parsed_args, body):
    max_kbps = parsed_args.max_kbps
    max_burst_kbps = parsed_args.max_burst_kbps
    if not (max_kbps or max_burst_kbps):
        raise exceptions.CommandError(_("Must provide max-kbps"
                                        " or max-burst-kbps option."))
    neutronv20.update_dict(parsed_args, body,
                           ['max_kbps', 'max_burst_kbps', 'tenant_id'])


class CreateQoSBandwidthLimitRule(qos_rule.QosRuleMixin,
                                  neutronv20.CreateCommand):
    """Create a qos bandwidth limit rule."""

    resource = BANDWIDTH_LIMIT_RULE_RESOURCE

    def add_known_arguments(self, parser):
        super(CreateQoSBandwidthLimitRule, self).add_known_arguments(parser)
        add_bandwidth_limit_arguments(parser)

    def args2body(self, parsed_args):
        body = {}
        update_bandwidth_limit_args2body(parsed_args, body)
        return {self.resource: body}


class ListQoSBandwidthLimitRules(qos_rule.QosRuleMixin,
                                 neutronv20.ListCommand):
    """List all qos bandwidth limit rules belonging to the specified policy."""

    resource = BANDWIDTH_LIMIT_RULE_RESOURCE
    _formatters = {}
    pagination_support = True
    sorting_support = True


class ShowQoSBandwidthLimitRule(qos_rule.QosRuleMixin, neutronv20.ShowCommand):
    """Show information about the given qos bandwidth limit rule."""

    resource = BANDWIDTH_LIMIT_RULE_RESOURCE
    allow_names = False


class UpdateQoSBandwidthLimitRule(qos_rule.QosRuleMixin,
                                  neutronv20.UpdateCommand):
    """Update the given qos bandwidth limit rule."""

    resource = BANDWIDTH_LIMIT_RULE_RESOURCE
    allow_names = False

    def add_known_arguments(self, parser):
        super(UpdateQoSBandwidthLimitRule, self).add_known_arguments(parser)
        add_bandwidth_limit_arguments(parser)

    def args2body(self, parsed_args):
        body = {}
        update_bandwidth_limit_args2body(parsed_args, body)
        return {self.resource: body}


class DeleteQoSBandwidthLimitRule(qos_rule.QosRuleMixin,
                                  neutronv20.DeleteCommand):
    """Delete a given qos bandwidth limit rule."""

    resource = BANDWIDTH_LIMIT_RULE_RESOURCE
    allow_names = False



# Copyright 2016-2017 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _
from neutronclient.osc import utils as osc_utils
from neutronclient.osc.v2.fwaas import constants as const
from neutronclient.osc.v2 import utils as v2_utils


LOG = logging.getLogger(__name__)

_formatters = {
    'admin_state_up': v2_utils.AdminStateColumn,
}

_attr_map = (
    ('id', 'ID', column_util.LIST_BOTH),
    ('name', 'Name', column_util.LIST_BOTH),
    ('ingress_firewall_policy_id', 'Ingress Policy ID', column_util.LIST_BOTH),
    ('egress_firewall_policy_id', 'Egress Policy ID', column_util.LIST_BOTH),
    ('description', 'Description', column_util.LIST_LONG_ONLY),
    ('status', 'Status', column_util.LIST_LONG_ONLY),
    ('ports', 'Ports', column_util.LIST_LONG_ONLY),
    ('admin_state_up', 'State', column_util.LIST_LONG_ONLY),
    ('shared', 'Shared', column_util.LIST_LONG_ONLY),
    ('tenant_id', 'Project', column_util.LIST_LONG_ONLY),
)


def _get_common_parser(parser):
    parser.add_argument(
        '--name',
        help=_('Name for the firewall group'))
    parser.add_argument(
        '--description',
        metavar='<description>',
        help=_('Description of the firewall group'))
    ingress_group = parser.add_mutually_exclusive_group()
    ingress_group.add_argument(
        '--ingress-firewall-policy',
        metavar='<ingress-firewall-policy>',
        dest='ingress_firewall_policy',
        help=_('Ingress firewall policy (name or ID)'))
    ingress_group.add_argument(
        '--no-ingress-firewall-policy',
        dest='no_ingress_firewall_policy',
        action='store_true',
        help=_('Detach ingress firewall policy from the firewall group'))
    egress_group = parser.add_mutually_exclusive_group()
    egress_group.add_argument(
        '--egress-firewall-policy',
        metavar='<egress-firewall-policy>',
        dest='egress_firewall_policy',
        help=_('Egress firewall policy (name or ID)'))
    egress_group.add_argument(
        '--no-egress-firewall-policy',
        dest='no_egress_firewall_policy',
        action='store_true',
        help=_('Detach egress firewall policy from the firewall group'))
    shared_group = parser.add_mutually_exclusive_group()

    shared_group.add_argument(
        '--share',
        action='store_true',
        help=_('Share the firewall group to be used in all projects '
               '(by default, it is restricted to be used by the '
               'current project).'))
    shared_group.add_argument(
        '--no-share',
        action='store_true',
        help=_('Restrict use of the firewall group to the '
               'current project'))
    admin_group = parser.add_mutually_exclusive_group()
    admin_group.add_argument(
        '--enable',
        action='store_true',
        help=_('Enable firewall group'))
    admin_group.add_argument(
        '--disable',
        action='store_true',
        help=_('Disable firewall group'))
    return parser


def _get_common_attrs(client_manager, parsed_args, is_create=True):
    attrs = {}
    client = client_manager.neutronclient

    if is_create:
        if 'project' in parsed_args and parsed_args.project is not None:
            attrs['tenant_id'] = osc_utils.find_project(
                client_manager.identity,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
    if (parsed_args.ingress_firewall_policy and
            parsed_args.no_ingress_firewall_policy):
        attrs['ingress_firewall_policy_id'] = client.find_resource(
            const.FWP, parsed_args.ingress_firewall_policy,
            cmd_resource=const.CMD_FWP)['id']
    elif parsed_args.ingress_firewall_policy:
        attrs['ingress_firewall_policy_id'] = client.find_resource(
            const.FWP, parsed_args.ingress_firewall_policy,
            cmd_resource=const.CMD_FWP)['id']
    elif parsed_args.no_ingress_firewall_policy:
        attrs['ingress_firewall_policy_id'] = None
    if (parsed_args.egress_firewall_policy and
            parsed_args.no_egress_firewall_policy):
        attrs['egress_firewall_policy_id'] = client.find_resource(
            const.FWP, parsed_args.egress_firewall_policy,
            cmd_resource=const.CMD_FWP)['id']
    elif parsed_args.egress_firewall_policy:
        attrs['egress_firewall_policy_id'] = client.find_resource(
            const.FWP, parsed_args.egress_firewall_policy,
            cmd_resource=const.CMD_FWP)['id']
    elif parsed_args.no_egress_firewall_policy:
        attrs['egress_firewall_policy_id'] = None
    if parsed_args.share:
        attrs['shared'] = True
    if parsed_args.no_share:
        attrs['shared'] = False
    if parsed_args.enable:
        attrs['admin_state_up'] = True
    if parsed_args.disable:
        attrs['admin_state_up'] = False
    if parsed_args.name:
        attrs['name'] = str(parsed_args.name)
    if parsed_args.description:
        attrs['description'] = str(parsed_args.description)
    if parsed_args.port and parsed_args.no_port:
        attrs['ports'] = sorted([client.find_resource(
            'port', p)['id'] for p in set(parsed_args.port)])
    elif parsed_args.port:
        ports = []
        for p in set(parsed_args.port):
            ports.append(client.find_resource('port', p)['id'])
        if not is_create:
            ports += client.find_resource(
                const.FWG, parsed_args.firewall_group,
                cmd_resource=const.CMD_FWG)['ports']
        attrs['ports'] = sorted(set(ports))
    elif parsed_args.no_port:
        attrs['ports'] = []
    return attrs


class CreateFirewallGroup(command.ShowOne):
    _description = _("Create a new firewall group")

    def get_parser(self, prog_name):
        parser = super(CreateFirewallGroup, self).get_parser(prog_name)
        _get_common_parser(parser)
        osc_utils.add_project_owner_option_to_parser(parser)
        port_group = parser.add_mutually_exclusive_group()
        port_group.add_argument(
            '--port',
            metavar='<port>',
            action='append',
            help=_('Port(s) (name or ID) to apply firewall group.  This '
                   'option can be repeated'))
        port_group.add_argument(
            '--no-port',
            dest='no_port',
            action='store_true',
            help=_('Detach all port from the firewall group'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_common_attrs(self.app.client_manager, parsed_args)
        obj = client.create_fwaas_firewall_group(
            {const.FWG: attrs})[const.FWG]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = utils.get_dict_properties(obj, columns, formatters=_formatters)
        return (display_columns, data)


class DeleteFirewallGroup(command.Command):
    _description = _("Delete firewall group(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteFirewallGroup, self).get_parser(prog_name)
        parser.add_argument(
            const.FWG,
            metavar='<firewall-group>',
            nargs='+',
            help=_('Firewall group(s) to delete (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        result = 0
        for fwg in parsed_args.firewall_group:
            try:
                fwg_id = client.find_resource(
                    const.FWG, fwg, cmd_resource=const.CMD_FWG)['id']
                client.delete_fwaas_firewall_group(fwg_id)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete firewall group with "
                            "name or ID '%(firewall_group)s': %(e)s"),
                          {const.FWG: fwg, 'e': e})

        if result > 0:
            total = len(parsed_args.firewall_group)
            msg = (_("%(result)s of %(total)s firewall group(s) "
                     "failed to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class ListFirewallGroup(command.Lister):
    _description = _("List firewall groups")

    def get_parser(self, prog_name):
        parser = super(ListFirewallGroup, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            help=_("List additional fields in output")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        obj = client.list_fwaas_firewall_groups()[const.FWGS]
        headers, columns = column_util.get_column_definitions(
            _attr_map, long_listing=parsed_args.long)
        return (headers, (utils.get_dict_properties(
            s, columns, formatters=_formatters) for s in obj))


class SetFirewallGroup(command.Command):
    _description = _("Set firewall group properties")

    def get_parser(self, prog_name):
        parser = super(SetFirewallGroup, self).get_parser(prog_name)
        _get_common_parser(parser)
        parser.add_argument(
            const.FWG,
            metavar='<firewall-group>',
            help=_('Firewall group to update (name or ID)'))
        parser.add_argument(
            '--port',
            metavar='<port>',
            action='append',
            help=_('Port(s) (name or ID) to apply firewall group.  This '
                   'option can be repeated'))
        parser.add_argument(
            '--no-port',
            dest='no_port',
            action='store_true',
            help=_('Detach all port from the firewall group'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        fwg_id = client.find_resource(const.FWG, parsed_args.firewall_group,
                                      cmd_resource=const.CMD_FWG)['id']
        attrs = _get_common_attrs(self.app.client_manager, parsed_args,
                                  is_create=False)
        try:
            client.update_fwaas_firewall_group(fwg_id, {const.FWG: attrs})
        except Exception as e:
            msg = (_("Failed to set firewall group '%(group)s': %(e)s")
                   % {'group': parsed_args.firewall_group, 'e': e})
            raise exceptions.CommandError(msg)


class ShowFirewallGroup(command.ShowOne):
    _description = _("Display firewall group details")

    def get_parser(self, prog_name):
        parser = super(ShowFirewallGroup, self).get_parser(prog_name)
        parser.add_argument(
            const.FWG,
            metavar='<firewall-group>',
            help=_('Firewall group to show (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        fwg_id = client.find_resource(const.FWG, parsed_args.firewall_group,
                                      cmd_resource=const.CMD_FWG)['id']
        obj = client.show_fwaas_firewall_group(fwg_id)[const.FWG]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = utils.get_dict_properties(obj, columns, formatters=_formatters)
        return (display_columns, data)


class UnsetFirewallGroup(command.Command):
    _description = _("Unset firewall group properties")

    def get_parser(self, prog_name):
        parser = super(UnsetFirewallGroup, self).get_parser(prog_name)
        parser.add_argument(
            const.FWG,
            metavar='<firewall-group>',
            help=_('Firewall group to unset (name or ID)'))
        port_group = parser.add_mutually_exclusive_group()
        port_group.add_argument(
            '--port',
            metavar='<port>',
            action='append',
            help=_('Port(s) (name or ID) to apply firewall group.  This '
                   'option can be repeated'))
        port_group.add_argument(
            '--all-port',
            action='store_true',
            help=_('Remove all ports for this firewall group'))
        parser.add_argument(
            '--ingress-firewall-policy',
            action='store_true',
            help=_('Ingress firewall policy (name or ID) to delete'))
        parser.add_argument(
            '--egress-firewall-policy',
            action='store_true',
            dest='egress_firewall_policy',
            help=_('Egress firewall policy (name or ID) to delete'))
        shared_group = parser.add_mutually_exclusive_group()
        shared_group.add_argument(
            '--share',
            action='store_true',
            help=_('Restrict use of the firewall group to the '
                   'current project'))
        parser.add_argument(
            '--enable',
            action='store_true',
            help=_('Disable firewall group'))
        return parser

    def _get_attrs(self, client_manager, parsed_args):
        attrs = {}
        client = client_manager.neutronclient
        if parsed_args.ingress_firewall_policy:
            attrs['ingress_firewall_policy_id'] = None
        if parsed_args.egress_firewall_policy:
            attrs['egress_firewall_policy_id'] = None
        if parsed_args.share:
            attrs['shared'] = False
        if parsed_args.enable:
            attrs['admin_state_up'] = False
        if parsed_args.port:
            old = client.find_resource(
                const.FWG, parsed_args.firewall_group,
                cmd_resource=const.CMD_FWG)['ports']
            new = [client.find_resource(
                'port', r)['id'] for r in parsed_args.port]
            attrs['ports'] = sorted(list(set(old) - set(new)))
        if parsed_args.all_port:
            attrs['ports'] = []
        return attrs

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        fwg_id = client.find_resource(const.FWG, parsed_args.firewall_group,
                                      cmd_resource=const.CMD_FWG)['id']
        attrs = self._get_attrs(self.app.client_manager, parsed_args)
        try:
            client.update_fwaas_firewall_group(fwg_id, {const.FWG: attrs})
        except Exception as e:
            msg = (_("Failed to unset firewall group '%(group)s': %(e)s")
                   % {'group': parsed_args.firewall_group, 'e': e})
            raise exceptions.CommandError(msg)



# Copyright 2014 Blue Box Group, Inc.
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from oslo_serialization import jsonutils

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


def _add_common_args(parser):
    parser.add_argument(
        '--description',
        help=_('Description of the load balancer.'))
    parser.add_argument(
        '--name', metavar='NAME',
        help=_('Name of the load balancer.'))


def _parse_common_args(body, parsed_args):
    neutronV20.update_dict(parsed_args, body,
                           ['name', 'description'])


class ListLoadBalancer(neutronV20.ListCommand):
    """LBaaS v2 List loadbalancers that belong to a given tenant."""

    resource = 'loadbalancer'
    list_columns = ['id', 'name', 'vip_address',
                    'provisioning_status', 'provider']
    pagination_support = True
    sorting_support = True


class ShowLoadBalancer(neutronV20.ShowCommand):
    """LBaaS v2 Show information of a given loadbalancer."""

    resource = 'loadbalancer'


class CreateLoadBalancer(neutronV20.CreateCommand):
    """LBaaS v2 Create a loadbalancer."""

    resource = 'loadbalancer'

    def add_known_arguments(self, parser):
        _add_common_args(parser)
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state', action='store_false',
            help=_('Set admin state up to false.'))
        parser.add_argument(
            '--provider',
            help=_('Provider name of the load balancer service.'))
        parser.add_argument(
            '--flavor',
            help=_('ID or name of the flavor.'))
        parser.add_argument(
            '--vip-address',
            help=_('VIP address for the load balancer.'))
        parser.add_argument(
            'vip_subnet', metavar='VIP_SUBNET',
            help=_('Load balancer VIP subnet.'))

    def args2body(self, parsed_args):
        _subnet_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'subnet', parsed_args.vip_subnet)
        body = {'vip_subnet_id': _subnet_id,
                'admin_state_up': parsed_args.admin_state}
        if parsed_args.flavor:
            _flavor_id = neutronV20.find_resourceid_by_name_or_id(
                self.get_client(), 'flavor', parsed_args.flavor)
            body['flavor_id'] = _flavor_id

        neutronV20.update_dict(parsed_args, body,
                               ['provider', 'vip_address', 'tenant_id'])
        _parse_common_args(body, parsed_args)
        return {self.resource: body}


class UpdateLoadBalancer(neutronV20.UpdateCommand):
    """LBaaS v2 Update a given loadbalancer."""

    resource = 'loadbalancer'

    def add_known_arguments(self, parser):
        utils.add_boolean_argument(
            parser, '--admin-state-up',
            help=_('Update the administrative state of '
                   'the load balancer (True meaning "Up").'))
        _add_common_args(parser)

    def args2body(self, parsed_args):
        body = {}
        _parse_common_args(body, parsed_args)
        neutronV20.update_dict(parsed_args, body,
                               ['admin_state_up'])
        return {self.resource: body}


class DeleteLoadBalancer(neutronV20.DeleteCommand):
    """LBaaS v2 Delete a given loadbalancer."""

    resource = 'loadbalancer'


class RetrieveLoadBalancerStats(neutronV20.ShowCommand):
    """Retrieve stats for a given loadbalancer."""

    resource = 'loadbalancer'

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        loadbalancer_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'loadbalancer', parsed_args.id)
        params = {}
        if parsed_args.fields:
            params = {'fields': parsed_args.fields}

        data = neutron_client.retrieve_loadbalancer_stats(loadbalancer_id,
                                                          **params)
        self.format_output_data(data)
        stats = data['stats']
        if 'stats' in data:
            # To render the output table like:
            # +--------------------+-------+
            # | Field              | Value |
            # +--------------------+-------+
            # | field1             | value1|
            # | field2             | value2|
            # | field3             | value3|
            # | ...                | ...   |
            # +--------------------+-------+
            # it has two columns and the Filed column is alphabetical,
            # here convert the data dict to the 1-1 vector format below:
            # [(field1, field2, field3, ...), (value1, value2, value3, ...)]
            return list(zip(*sorted(stats.items())))


class RetrieveLoadBalancerStatus(neutronV20.NeutronCommand):
    """Retrieve status for a given loadbalancer.

    The only output is a formatted JSON tree, and the table format
    does not support this type of data.
    """
    resource = 'loadbalancer'

    def get_parser(self, prog_name):
        parser = super(RetrieveLoadBalancerStatus, self).get_parser(prog_name)
        parser.add_argument(
            self.resource, metavar=self.resource.upper(),
            help=_('ID or name of %s to show.') % self.resource)

        return parser

    def take_action(self, parsed_args):
        self.log.debug('run(%s)', parsed_args)
        neutron_client = self.get_client()
        lb_id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, self.resource, parsed_args.loadbalancer)
        params = {}
        data = neutron_client.retrieve_loadbalancer_status(lb_id, **params)
        res = data['statuses']
        if 'statuses' in data:
            print(jsonutils.dumps(res, indent=4))



# Copyright 2016 NEC Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutronclient.tests.functional import base


class CLICommonFeatureTest(base.ClientTestBase):

    def test_tenant_id_shown_in_list_by_admin(self):
        nets = self.parser.table(self.neutron('net-list'))
        self.assertIn('tenant_id', nets['headers'])

    def test_tenant_id_not_shown_in_list_with_columns(self):
        nets = self.parser.table(self.neutron('net-list -c id -c name'))
        self.assertNotIn('tenant_id', nets['headers'])
        self.assertListEqual(['id', 'name'], nets['headers'])

    def test_tenant_id_not_shown_in_list_by_non_admin(self):
        output = self.neutron_non_admin('net-list')
        self.assertNotIn('tenant_id', self.parser.table(output)['headers'])
        self.assertTableStruct(self.parser.listing(output),
                               ['id', 'name'])



#    (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import argparse

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronv20
from neutronclient.neutron.v2_0.vpn import utils as vpn_utils


def add_common_args(parser, is_create=True):
    parser.add_argument(
        '--auth-algorithm',
        default='sha1' if is_create else argparse.SUPPRESS,
        type=utils.convert_to_lowercase,
        choices=['sha1', 'sha256', 'sha384', 'sha512'],
        help=_('Authentication algorithm for IPsec policy, default:sha1.'))
    parser.add_argument(
        '--description',
        help=_('Description of the IPsec policy.'))
    parser.add_argument(
        '--encapsulation-mode',
        default='tunnel' if is_create else argparse.SUPPRESS,
        choices=['tunnel', 'transport'],
        type=utils.convert_to_lowercase,
        help=_('Encapsulation mode for IPsec policy, default:tunnel.'))
    parser.add_argument(
        '--encryption-algorithm',
        default='aes-128' if is_create else argparse.SUPPRESS,
        type=utils.convert_to_lowercase,
        help=_('Encryption algorithm for IPsec policy, default:aes-128.'))
    parser.add_argument(
        '--lifetime',
        metavar="units=UNITS,value=VALUE",
        type=utils.str2dict_type(optional_keys=['units', 'value']),
        help=vpn_utils.lifetime_help("IPsec"))
    parser.add_argument(
        '--pfs',
        default='group5' if is_create else argparse.SUPPRESS,
        type=utils.convert_to_lowercase,
        help=_('Perfect Forward Secrecy for IPsec policy, default:group5.'))
    parser.add_argument(
        '--transform-protocol',
        default='esp' if is_create else argparse.SUPPRESS,
        type=utils.convert_to_lowercase,
        choices=['esp', 'ah', 'ah-esp'],
        help=_('Transform protocol for IPsec policy, default:esp.'))


def parse_common_args2body(parsed_args, body):
    neutronv20.update_dict(parsed_args, body,
                           ['auth_algorithm', 'encryption_algorithm',
                            'encapsulation_mode', 'transform_protocol',
                            'pfs', 'name', 'description', 'tenant_id'])
    if parsed_args.lifetime:
        vpn_utils.validate_lifetime_dict(parsed_args.lifetime)
        body['lifetime'] = parsed_args.lifetime
    return body


class ListIPsecPolicy(neutronv20.ListCommand):
    """List IPsec policies that belong to a given tenant connection."""

    resource = 'ipsecpolicy'
    list_columns = ['id', 'name', 'auth_algorithm',
                    'encryption_algorithm', 'pfs']
    _formatters = {}
    pagination_support = True
    sorting_support = True


class ShowIPsecPolicy(neutronv20.ShowCommand):
    """Show information of a given IPsec policy."""

    resource = 'ipsecpolicy'
    help_resource = 'IPsec policy'


class CreateIPsecPolicy(neutronv20.CreateCommand):
    """Create an IPsec policy."""

    resource = 'ipsecpolicy'
    help_resource = 'IPsec policy'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of the IPsec policy.'))
        add_common_args(parser)

    def args2body(self, parsed_args):
        return {'ipsecpolicy': parse_common_args2body(parsed_args, body={})}


class UpdateIPsecPolicy(neutronv20.UpdateCommand):
    """Update a given IPsec policy."""

    resource = 'ipsecpolicy'
    help_resource = 'IPsec policy'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            help=_('Updated name of the IPsec policy.'))
        add_common_args(parser, is_create=False)

    def args2body(self, parsed_args):
        return {'ipsecpolicy': parse_common_args2body(parsed_args, body={})}


class DeleteIPsecPolicy(neutronv20.DeleteCommand):
    """Delete a given IPsec policy."""

    resource = 'ipsecpolicy'
    help_resource = 'IPsec policy'



# Copyright 2013 Mirantis Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronV20


class ListMember(neutronV20.ListCommand):
    """List members that belong to a given tenant."""

    resource = 'member'
    list_columns = [
        'id', 'address', 'protocol_port', 'weight', 'admin_state_up', 'status'
    ]
    pagination_support = True
    sorting_support = True


class ShowMember(neutronV20.ShowCommand):
    """Show information of a given member."""

    resource = 'member'
    allow_names = False


class CreateMember(neutronV20.CreateCommand):
    """Create a member."""

    resource = 'member'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state', action='store_false',
            help=_('Set admin state up to false.'))
        parser.add_argument(
            '--weight',
            help=_('Weight of pool member in the pool (default:1, [0..256]).'))
        parser.add_argument(
            '--address',
            required=True,
            help=_('IP address of the pool member on the pool network.'))
        parser.add_argument(
            '--protocol-port',
            required=True,
            help=_('Port on which the pool member listens for requests or '
                   'connections.'))
        parser.add_argument(
            'pool_id', metavar='POOL',
            help=_('ID or name of the pool this vip belongs to.'))

    def args2body(self, parsed_args):
        _pool_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'pool', parsed_args.pool_id)
        body = {'pool_id': _pool_id,
                'admin_state_up': parsed_args.admin_state}
        neutronV20.update_dict(
            parsed_args,
            body,
            ['address', 'protocol_port', 'weight', 'tenant_id']
        )
        return {self.resource: body}


class UpdateMember(neutronV20.UpdateCommand):
    """Update a given member."""

    resource = 'member'


class DeleteMember(neutronV20.DeleteCommand):
    """Delete a given member."""

    resource = 'member'



# Copyright 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.flavor import flavor_profile
from neutronclient.tests.unit import test_cli20


class CLITestV20FlavorProfileJSON(test_cli20.CLITestV20Base):

    def setUp(self):
        """Prepare test environment."""
        super(CLITestV20FlavorProfileJSON, self).setUp(
            plurals={'service_profiles': 'service_profile'})
        self.register_non_admin_status_resource('service_profile')

    def test_create_flavor_profile_with_mandatory_params(self):
        """Create test flavor profile test."""
        resource = 'service_profile'
        cmd = flavor_profile.CreateFlavorProfile(
            test_cli20.MyApp(sys.stdout), None)
        name = ''
        description = 'Test flavor profile'
        myid = 'myid'
        metainfo = "{'a':'b'}"
        # Defaults are returned in body
        position_names = ['description', 'metainfo']
        position_values = [description, metainfo]
        args = ['--description', description, '--metainfo', metainfo]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_flavor_profile_with_optional_params(self):
        """Create test flavor profile disabled test."""
        resource = 'service_profile'
        cmd = flavor_profile.CreateFlavorProfile(
            test_cli20.MyApp(sys.stdout), None)
        name = ''
        description = 'Test flavor profile - disabled'
        myid = 'myid'
        driver = 'mydriver'
        metainfo = "{'a':'b'}"
        position_names = ['description', 'driver', 'metainfo', 'enabled']
        position_values = [description, driver, metainfo, 'False']
        args = ['--description', description, '--driver', driver,
                '--metainfo', metainfo, '--enabled=False']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_list_flavor_profiles(self):
        """List flavor profiles test."""
        resources = 'service_profiles'
        cmd = flavor_profile.ListFlavorProfile(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_flavor_profiles_with_pagination(self):
        """List flavor profiles test with pagination."""
        resources = 'service_profiles'
        cmd = flavor_profile.ListFlavorProfile(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_flavor_profiles_with_sort(self):
        """List flavor profiles test with sort by description."""
        resources = 'service_profiles'
        cmd = flavor_profile.ListFlavorProfile(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["description"],
                                  sort_dir=["asc"])

    def test_show_flavor_profile(self):
        """Show flavor profile test."""
        resource = 'service_profile'
        cmd = flavor_profile.ShowFlavorProfile(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_update_flavor_profile(self):
        """Update flavor profile test."""
        resource = 'service_profile'
        cmd = flavor_profile.UpdateFlavorProfile(
            test_cli20.MyApp(sys.stdout), None)
        newdescription = 'Test new description'
        newdriver = 'NewDriver'
        newmetainfo = "{'c':'d'}"
        newenabled = "False"
        args = ['--description', newdescription,
                '--driver', newdriver,
                '--metainfo', newmetainfo,
                '--enabled', newenabled,
                self.test_id]
        self._test_update_resource(resource, cmd, self.test_id, args,
                                   {'description': newdescription,
                                    'driver': newdriver,
                                    'metainfo': newmetainfo,
                                    'enabled': newenabled})

    def test_delete_flavor_profile(self):
        """Delete flavor profile."""
        resource = 'service_profile'
        cmd = flavor_profile.DeleteFlavorProfile(test_cli20.MyApp(sys.stdout),
                                                 None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



# Copyright 2016 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import copy
import re
from unittest import mock

from osc_lib import exceptions
from osc_lib.tests import utils
import testtools

from neutronclient.osc import utils as osc_utils
from neutronclient.osc.v2.fwaas import constants as const
from neutronclient.osc.v2.fwaas import firewallrule
from neutronclient.tests.unit.osc.v2 import fakes as test_fakes
from neutronclient.tests.unit.osc.v2.fwaas import common
from neutronclient.tests.unit.osc.v2.fwaas import fakes


_fwr = fakes.FirewallRule().create()
CONVERT_MAP = {
    'project': 'tenant_id',
    'enable_rule': 'enabled',
    'disable_rule': 'enabled',
    'share': 'shared',
    'no_share': 'shared',
    'source_firewall_group': 'source_firewall_group_id',
    'destination_firewall_group': 'destination_firewall_group_id',
    'no_source_firewall_group': 'source_firewall_group_id',
    'no_destination_firewall_group': 'destination_firewall_group_id',
}


def _generate_data(ordered_dict=None, data=None):
    source = ordered_dict if ordered_dict else _fwr
    if data:
        source.update(data)
    return tuple(_replace_display_columns(key, source[key]) for key in source)


def _replace_display_columns(key, val):
    if key == 'protocol':
        return firewallrule.ProtocolColumn(val)
    return val


def _generate_req_and_res(verifylist):
    request = dict(verifylist)
    response = copy.deepcopy(_fwr)
    for key, val in verifylist:
        converted = CONVERT_MAP.get(key, key)
        del request[key]
        if re.match('^no_', key) and val is True:
            new_value = None
        elif (key == 'enable' or key == 'enable_rule') and val:
            new_value = True
        elif (key == 'disable' or key == 'disable_rule') and val:
            new_value = False
        elif (key == 'protocol' and val and val.lower() == 'any'):
            new_value = None
        else:
            new_value = val
        request[converted] = new_value
        response[converted] = new_value
    return request, response


class TestFirewallRule(test_fakes.TestNeutronClientOSCV2):

    def check_results(self, headers, data, exp_req, is_list=False):
        if is_list:
            req_body = {self.res_plural: [exp_req]}
        else:
            req_body = {self.res: exp_req}
        self.mocked.assert_called_once_with(req_body)
        self.assertEqual(self.ordered_headers, headers)
        self.assertItemEqual(self.ordered_data, data)

    def setUp(self):
        super(TestFirewallRule, self).setUp()

        def _mock_fwr(*args, **kwargs):
            self.neutronclient.find_resource.assert_called_once_with(
                self.res, self.resource['id'], cmd_resource=const.CMD_FWR)
            return {'id': args[1]}

        self.neutronclient.find_resource.side_effect = mock.Mock(
            side_effect=_mock_fwr)
        osc_utils.find_project = mock.Mock()
        osc_utils.find_project.id = _fwr['tenant_id']
        self.res = 'firewall_rule'
        self.res_plural = 'firewall_rules'
        self.resource = _fwr
        self.headers = (
            'ID',
            'Name',
            'Enabled',
            'Description',
            'IP Version',
            'Action',
            'Protocol',
            'Source IP Address',
            'Source Port',
            'Destination IP Address',
            'Destination Port',
            'Shared',
            'Project',
            'Source Firewall Group ID',
            'Destination Firewall Group ID',
        )
        self.data = _generate_data()
        self.ordered_headers = (
            'Action',
            'Description',
            'Destination Firewall Group ID',
            'Destination IP Address',
            'Destination Port',
            'Enabled',
            'ID',
            'IP Version',
            'Name',
            'Project',
            'Protocol',
            'Shared',
            'Source Firewall Group ID',
            'Source IP Address',
            'Source Port',
        )
        self.ordered_data = (
            _fwr['action'],
            _fwr['description'],
            _fwr['destination_firewall_group_id'],
            _fwr['destination_ip_address'],
            _fwr['destination_port'],
            _fwr['enabled'],
            _fwr['id'],
            _fwr['ip_version'],
            _fwr['name'],
            _fwr['tenant_id'],
            _replace_display_columns('protocol', _fwr['protocol']),
            _fwr['shared'],
            _fwr['source_firewall_group_id'],
            _fwr['source_ip_address'],
            _fwr['source_port'],
        )
        self.ordered_columns = (
            'action',
            'description',
            'destination_firewall_group_id',
            'destination_ip_address',
            'destination_port',
            'enabled',
            'id',
            'ip_version',
            'name',
            'tenant_id',
            'protocol',
            'shared',
            'source_firewall_group_id',
            'source_ip_address',
            'source_port',
        )


class TestCreateFirewallRule(TestFirewallRule, common.TestCreateFWaaS):

    def setUp(self):
        super(TestCreateFirewallRule, self).setUp()
        self.neutronclient.create_fwaas_firewall_rule = mock.Mock(
            return_value={self.res: _fwr})
        self.mocked = self.neutronclient.create_fwaas_firewall_rule
        self.cmd = firewallrule.CreateFirewallRule(self.app, self.namespace)

    def _update_expect_response(self, request, response):
        """Set expected request and response

        :param request
            A dictionary of request body(dict of verifylist)
        :param response
            A OrderedDict of request body
        """
        # Update response body
        self.neutronclient.create_fwaas_firewall_rule.return_value = \
            {self.res: dict(response)}
        osc_utils.find_project.return_value.id = response['tenant_id']
        # Update response(finally returns 'data')
        self.data = _generate_data(ordered_dict=response)
        self.ordered_data = tuple(
            _replace_display_columns(column, response[column])
            for column in self.ordered_columns
        )

    def _set_all_params(self, args={}):
        name = args.get('name') or 'my-name'
        description = args.get('description') or 'my-desc'
        source_ip = args.get('source_ip_address') or '192.168.1.0/24'
        destination_ip = args.get('destination_ip_address') or '192.168.2.0/24'
        source_port = args.get('source_port') or '0:65535'
        protocol = args.get('protocol') or 'udp'
        action = args.get('action') or 'deny'
        ip_version = args.get('ip_version') or '4'
        destination_port = args.get('destination_port') or '0:65535'
        destination_firewall_group = args.get(
            'destination_firewall_group') or 'my-dst-fwg'
        source_firewall_group = args.get(
            'source_firewall_group') or 'my-src-fwg'
        tenant_id = args.get('tenant_id') or 'my-tenant'
        arglist = [
            '--description', description,
            '--name', name,
            '--protocol', protocol,
            '--ip-version', ip_version,
            '--source-ip-address', source_ip,
            '--destination-ip-address', destination_ip,
            '--source-port', source_port,
            '--destination-port', destination_port,
            '--action', action,
            '--project', tenant_id,
            '--disable-rule',
            '--share',
            '--source-firewall-group', source_firewall_group,
            '--destination-firewall-group', destination_firewall_group
        ]

        verifylist = [
            ('name', name),
            ('description', description),
            ('share', True),
            ('protocol', protocol),
            ('ip_version', ip_version),
            ('source_ip_address', source_ip),
            ('destination_ip_address', destination_ip),
            ('source_port', source_port),
            ('destination_port', destination_port),
            ('action', action),
            ('disable_rule', True),
            ('project', tenant_id),
            ('source_firewall_group', source_firewall_group),
            ('destination_firewall_group', destination_firewall_group)
        ]
        return arglist, verifylist

    def _test_create_with_all_params(self, args={}):
        def _mock_fwr(*args, **kwargs):
            if self.neutronclient.find_resource.call_count == 1:
                self.neutronclient.find_resource.assert_called_once_with(
                    const.FWG, 'my-src-fwg', cmd_resource=const.CMD_FWG)
            if self.neutronclient.find_resource.call_count == 2:
                self.neutronclient.find_resource.assert_called_with(
                    const.FWG, 'my-dst-fwg', cmd_resource=const.CMD_FWG)
            return {'id': args[1]}

        self.neutronclient.find_resource.side_effect = mock.Mock(
            side_effect=_mock_fwr)
        arglist, verifylist = self._set_all_params(args)
        request, response = _generate_req_and_res(verifylist)
        self._update_expect_response(request, response)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.check_results(headers, data, request)

    def test_create_with_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)
        self.check_results(headers, data, {})

    def test_create_with_all_params(self):
        self._test_create_with_all_params()

    def test_create_with_all_params_protocol_any(self):
        self._test_create_with_all_params({'protocol': 'any'})

    def test_create_with_all_params_ip_version_6(self):
        self._test_create_with_all_params({'ip_version': '6'})

    def test_create_with_all_params_invalid_ip_version(self):
        arglist, verifylist = self._set_all_params({'ip_version': '128'})
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_all_params_action_upper_capitalized(self):
        for action in ('Allow', 'DENY', 'Reject'):
            arglist, verifylist = self._set_all_params({'action': action})
            self.assertRaises(
                testtools.matchers._impl.MismatchError,
                self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_all_params_protocol_upper_capitalized(self):
        for protocol in ('TCP', 'Tcp', 'ANY', 'AnY', 'iCMp'):
            arglist, verifylist = self._set_all_params({'protocol': protocol})
            self.assertRaises(
                testtools.matchers._impl.MismatchError,
                self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_src_fwg_and_no(self):
        fwg = 'my-fwg'
        arglist = [
            '--source-firewall-group', fwg,
            '--no-source-firewall-group',
        ]
        verifylist = [
            ('source_firewall_group', fwg),
            ('no_source_firewall_group', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_dst_fwg_and_no(self):
        fwg = 'my-fwg'
        arglist = [
            '--destination-firewall-group', fwg,
            '--no-destination-firewall-group',
        ]
        verifylist = [
            ('destination_firewall_group', fwg),
            ('no_destination_firewall_group', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)


class TestListFirewallRule(TestFirewallRule):

    def _setup_summary(self, expect=None):
        protocol = (_fwr['protocol'] or 'any').upper()
        src = 'source(port): 192.168.1.0/24(1:11111)'
        dst = 'dest(port): 192.168.2.2(2:22222)'
        action = 'deny'
        if expect:
            if expect.get('protocol'):
                protocol = expect['protocol']
            if expect.get('source'):
                src = expect['source']
            if expect.get('dest'):
                dst = expect['dest']
            if expect.get('action'):
                action = expect['action']
        summary = ',\n '.join([protocol, src, dst, action])
        self.short_data = (
            _fwr['id'],
            _fwr['name'],
            _fwr['enabled'],
            summary
        )

    def setUp(self):
        super(TestListFirewallRule, self).setUp()
        self.cmd = firewallrule.ListFirewallRule(self.app, self.namespace)

        self.short_header = (
            'ID',
            'Name',
            'Enabled',
            'Summary',
        )
        self._setup_summary()
        self.neutronclient.list_fwaas_firewall_rules = mock.Mock(
            return_value={self.res_plural: [_fwr]})
        self.mocked = self.neutronclient.list_fwaas_firewall_rules

    def test_list_with_long_option(self):
        arglist = ['--long']
        verifylist = [('long', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.headers), headers)
        m = list(data)
        self.assertListItemEqual([self.data], m)

    def test_list_with_no_option(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.short_header), headers)
        self.assertListItemEqual([self.short_data], list(data))


class TestShowFirewallRule(TestFirewallRule, common.TestShowFWaaS):

    def setUp(self):
        super(TestShowFirewallRule, self).setUp()
        self.neutronclient.show_fwaas_firewall_rule = mock.Mock(
            return_value={self.res: _fwr})
        self.mocked = self.neutronclient.show_fwaas_firewall_rule
        self.cmd = firewallrule.ShowFirewallRule(self.app, self.namespace)


class TestSetFirewallRule(TestFirewallRule, common.TestSetFWaaS):

    def setUp(self):
        super(TestSetFirewallRule, self).setUp()
        self.neutronclient.update_fwaas_firewall_rule = mock.Mock(
            return_value={self.res: _fwr})
        self.mocked = self.neutronclient.update_fwaas_firewall_rule
        self.cmd = firewallrule.SetFirewallRule(self.app, self.namespace)

    def test_set_protocol_with_any(self):
        target = self.resource['id']
        protocol = 'any'
        arglist = [target, '--protocol', protocol]
        verifylist = [
            (self.res, target),
            ('protocol', protocol),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'protocol': None}})
        self.assertIsNone(result)

    def test_set_protocol_with_udp(self):
        target = self.resource['id']
        protocol = 'udp'
        arglist = [target, '--protocol', protocol]
        verifylist = [
            (self.res, target),
            ('protocol', protocol),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'protocol': protocol}})
        self.assertIsNone(result)

    def test_set_source_ip_address(self):
        target = self.resource['id']
        src_ip = '192.192.192.192'
        arglist = [target, '--source-ip-address', src_ip]
        verifylist = [
            (self.res, target),
            ('source_ip_address', src_ip),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'source_ip_address': src_ip}})
        self.assertIsNone(result)

    def test_set_source_port(self):
        target = self.resource['id']
        src_port = '32678'
        arglist = [target, '--source-port', src_port]
        verifylist = [
            (self.res, target),
            ('source_port', src_port),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'source_port': src_port}})
        self.assertIsNone(result)

    def test_set_destination_ip_address(self):
        target = self.resource['id']
        dst_ip = '0.1.0.1'
        arglist = [target, '--destination-ip-address', dst_ip]
        verifylist = [
            (self.res, target),
            ('destination_ip_address', dst_ip),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'destination_ip_address': dst_ip}})
        self.assertIsNone(result)

    def test_set_destination_port(self):
        target = self.resource['id']
        dst_port = '65432'
        arglist = [target, '--destination-port', dst_port]
        verifylist = [
            (self.res, target),
            ('destination_port', dst_port),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'destination_port': dst_port}})
        self.assertIsNone(result)

    def test_set_enable_rule(self):
        target = self.resource['id']
        arglist = [target, '--enable-rule']
        verifylist = [
            (self.res, target),
            ('enable_rule', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'enabled': True}})
        self.assertIsNone(result)

    def test_set_disable_rule(self):
        target = self.resource['id']
        arglist = [target, '--disable-rule']
        verifylist = [
            (self.res, target),
            ('disable_rule', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'enabled': False}})
        self.assertIsNone(result)

    def test_set_action(self):
        target = self.resource['id']
        action = 'reject'
        arglist = [target, '--action', action]
        verifylist = [
            (self.res, target),
            ('action', action),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'action': action}})
        self.assertIsNone(result)

    def test_set_enable_rule_and_disable_rule(self):
        target = self.resource['id']
        arglist = [target, '--enable-rule', '--disable-rule']
        verifylist = [
            (self.res, target),
            ('enable_rule', True),
            ('disable_rule', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_set_no_source_ip_address(self):
        target = self.resource['id']
        arglist = [
            target,
            '--no-source-ip-address',
        ]
        verifylist = [
            (self.res, target),
            ('no_source_ip_address', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'source_ip_address': None}})
        self.assertIsNone(result)

    def test_set_no_source_port(self):
        target = self.resource['id']
        arglist = [
            target,
            '--no-source-port',
        ]
        verifylist = [
            (self.res, target),
            ('no_source_port', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'source_port': None}})
        self.assertIsNone(result)

    def test_set_no_destination_ip_address(self):
        target = self.resource['id']
        arglist = [
            target,
            '--no-destination-ip-address',
        ]
        verifylist = [
            (self.res, target),
            ('no_destination_ip_address', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'destination_ip_address': None}})
        self.assertIsNone(result)

    def test_set_no_destination_port(self):
        target = self.resource['id']
        arglist = [
            target,
            '--no-destination-port',
        ]
        verifylist = [
            (self.res, target),
            ('no_destination_port', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'destination_port': None}})
        self.assertIsNone(result)

    def test_set_source_ip_address_and_no(self):
        target = self.resource['id']
        arglist = [
            target,
            '--source-ip-address', '192.168.1.0/24',
            '--no-source-ip-address',
        ]
        verifylist = [
            (self.res, target),
            ('source_ip_address', '192.168.1.0/24'),
            ('no_source_ip_address', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_set_destination_ip_address_and_no(self):
        target = self.resource['id']
        arglist = [
            target,
            '--destination-ip-address', '192.168.2.0/24',
            '--no-destination-ip-address',
        ]
        verifylist = [
            (self.res, target),
            ('destination_ip_address', '192.168.2.0/24'),
            ('no_destination_ip_address', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_set_source_port_and_no(self):
        target = self.resource['id']
        arglist = [
            target,
            '--source-port', '1:12345',
            '--no-source-port',
        ]
        verifylist = [
            (self.res, target),
            ('source_port', '1:12345'),
            ('no_source_port', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_set_destination_port_and_no(self):
        target = self.resource['id']
        arglist = [
            target,
            '--destination-port', '1:54321',
            '--no-destination-port',
        ]
        verifylist = [
            (self.res, target),
            ('destination_port', '1:54321'),
            ('no_destination_port', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_set_and_raises(self):
        self.neutronclient.update_fwaas_firewall_rule = mock.Mock(
            side_effect=Exception)
        target = self.resource['id']
        arglist = [target, '--name', 'my-name']
        verifylist = [(self.res, target), ('name', 'my-name')]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_set_no_destination_fwg(self):
        target = self.resource['id']
        arglist = [
            target,
            '--no-destination-firewall-group',
        ]
        verifylist = [
            (self.res, target),
            ('no_destination_firewall_group', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'destination_firewall_group_id': None}})
        self.assertIsNone(result)

    def test_set_no_source_fwg(self):
        target = self.resource['id']
        arglist = [
            target,
            '--no-source-firewall-group',
        ]
        verifylist = [
            (self.res, target),
            ('no_source_firewall_group', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'source_firewall_group_id': None}})
        self.assertIsNone(result)

    def test_create_with_src_fwg_and_no(self):
        target = self.resource['id']
        fwg = 'my-fwg'
        arglist = [
            target,
            '--source-firewall-group', fwg,
            '--no-source-firewall-group',
        ]
        verifylist = [
            (self.res, target),
            ('source_firewall_group', fwg),
            ('no_source_firewall_group', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_dst_fwg_and_no(self):
        target = self.resource['id']
        fwg = 'my-fwg'
        arglist = [
            target,
            '--destination-firewall-group', fwg,
            '--no-destination-firewall-group',
        ]
        verifylist = [
            (self.res, target),
            ('destination_firewall_group', fwg),
            ('no_destination_firewall_group', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)


class TestUnsetFirewallRule(TestFirewallRule, common.TestUnsetFWaaS):

    def setUp(self):
        super(TestUnsetFirewallRule, self).setUp()
        self.neutronclient.update_fwaas_firewall_rule = mock.Mock(
            return_value={self.res: _fwr})
        self.mocked = self.neutronclient.update_fwaas_firewall_rule
        self.cmd = firewallrule.UnsetFirewallRule(self.app, self.namespace)

    def test_unset_protocol_and_raise(self):
        self.neutronclient.update_fwaas_firewall_rule.side_effect = Exception
        target = self.resource['id']
        arglist = [
            target,
            '--protocol',
        ]
        verifylist = [
            (self.res, target),
            ('protocol', False)
        ]
        self.assertRaises(utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_unset_source_port(self):
        target = self.resource['id']
        arglist = [
            target,
            '--source-port',
        ]
        verifylist = [
            (self.res, target),
            ('source_port', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'source_port': None}})
        self.assertIsNone(result)

    def test_unset_destination_port(self):
        target = self.resource['id']
        arglist = [
            target,
            '--destination-port',
        ]
        verifylist = [
            (self.res, target),
            ('destination_port', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'destination_port': None}})
        self.assertIsNone(result)

    def test_unset_source_ip_address(self):
        target = self.resource['id']
        arglist = [
            target,
            '--source-ip-address',
        ]
        verifylist = [
            (self.res, target),
            ('source_ip_address', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'source_ip_address': None}})
        self.assertIsNone(result)

    def test_unset_destination_ip_address(self):
        target = self.resource['id']
        arglist = [
            target,
            '--destination-ip-address',
        ]
        verifylist = [
            (self.res, target),
            ('destination_ip_address', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'destination_ip_address': None}})
        self.assertIsNone(result)

    def test_unset_enable_rule(self):
        target = self.resource['id']
        arglist = [
            target,
            '--enable-rule',
        ]
        verifylist = [
            (self.res, target),
            ('enable_rule', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {self.res: {'enabled': False}})
        self.assertIsNone(result)


class TestDeleteFirewallRule(TestFirewallRule, common.TestDeleteFWaaS):

    def setUp(self):
        super(TestDeleteFirewallRule, self).setUp()
        self.neutronclient.delete_fwaas_firewall_rule = mock.Mock(
            return_value={self.res: _fwr})
        self.mocked = self.neutronclient.delete_fwaas_firewall_rule
        self.cmd = firewallrule.DeleteFirewallRule(self.app, self.namespace)



# Copyright 2014 Blue Box Group, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.common import exceptions
from neutronclient.neutron.v2_0.lb.v2 import listener
from neutronclient.tests.unit import test_cli20


class CLITestV20LbListenerJSON(test_cli20.CLITestV20Base):

    def test_create_listener_with_loadbalancer(self):
        # lbaas-listener-create with --loadbalancer
        resource = 'listener'
        cmd_resource = 'lbaas_listener'
        cmd = listener.CreateListener(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        loadbalancer_id = 'loadbalancer'
        protocol = 'TCP'
        protocol_port = '80'
        args = ['--protocol', protocol, '--protocol-port', protocol_port,
                '--loadbalancer', loadbalancer_id]
        position_names = ['protocol', 'protocol_port', 'loadbalancer_id']
        position_values = [protocol, protocol_port, loadbalancer_id,
                           True]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource)

    def test_create_listener_with_default_pool(self):
        # lbaas-listener-create with --default-pool and no --loadbalancer.
        resource = 'listener'
        cmd_resource = 'lbaas_listener'
        cmd = listener.CreateListener(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        default_pool_id = 'default-pool'
        protocol = 'TCP'
        protocol_port = '80'
        args = ['--protocol', protocol, '--protocol-port', protocol_port,
                '--default-pool', default_pool_id]
        position_names = ['protocol', 'protocol_port', 'default_pool_id']
        position_values = [protocol, protocol_port, default_pool_id,
                           True]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource)

    def test_create_listener_with_no_loadbalancer_or_default_pool(self):
        # lbaas-listener-create without --default-pool or --loadbalancer.
        resource = 'listener'
        cmd_resource = 'lbaas_listener'
        cmd = listener.CreateListener(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        protocol = 'TCP'
        protocol_port = '80'
        args = ['--protocol', protocol, '--protocol-port', protocol_port]
        position_names = ['protocol', 'protocol_port']
        position_values = [protocol, protocol_port, True]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource,
                                   no_api_call=True,
                                   expected_exception=exceptions.CommandError)

    def test_create_listener_with_all_params(self):
        # lbaas-listener-create with all params set.
        resource = 'listener'
        cmd_resource = 'lbaas_listener'
        cmd = listener.CreateListener(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        loadbalancer = 'loadbalancer'
        default_pool_id = 'default-pool'
        protocol = 'TCP'
        protocol_port = '80'
        connection_limit = 10
        def_tls_cont_ref = '11111'
        args = ['--admin-state-down',
                '--protocol', protocol, '--protocol-port', protocol_port,
                '--loadbalancer', loadbalancer,
                '--default-pool', default_pool_id,
                '--default-tls-container-ref', def_tls_cont_ref,
                '--sni-container-refs', '1111', '2222', '3333',
                '--connection-limit', '10']
        position_names = ['admin_state_up',
                          'protocol', 'protocol_port', 'loadbalancer_id',
                          'default_pool_id',
                          'default_tls_container_ref', 'sni_container_refs',
                          'connection_limit']
        position_values = [False, protocol, protocol_port, loadbalancer,
                           default_pool_id,
                           def_tls_cont_ref, ['1111', '2222', '3333'],
                           connection_limit]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource)

    def test_list_listeners(self):
        # lbaas-listener-list.
        resources = 'listeners'
        cmd_resources = 'lbaas_listeners'
        cmd = listener.ListListener(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True,
                                  cmd_resources=cmd_resources)

    def test_list_listeners_pagination(self):
        # lbaas-listener-list with pagination.
        resources = 'listeners'
        cmd_resources = 'lbaas_listeners'
        cmd = listener.ListListener(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd,
                                                  cmd_resources=cmd_resources)

    def test_list_listeners_sort(self):
        # lbaas-listener-list --sort-key id --sort-key asc.
        resources = 'listeners'
        cmd_resources = 'lbaas_listeners'
        cmd = listener.ListListener(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True,
                                  cmd_resources=cmd_resources)

    def test_list_listeners_limit(self):
        # lbaas-listener-list -P.
        resources = 'listeners'
        cmd_resources = 'lbaas_listeners'
        cmd = listener.ListListener(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000,
                                  cmd_resources=cmd_resources)

    def test_show_listener_id(self):
        # lbaas-listener-show test_id.
        resource = 'listener'
        cmd_resource = 'lbaas_listener'
        cmd = listener.ShowListener(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'],
                                 cmd_resource=cmd_resource)

    def test_show_listener_id_name(self):
        # lbaas-listener-show.
        resource = 'listener'
        cmd_resource = 'lbaas_listener'
        cmd = listener.ShowListener(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'],
                                 cmd_resource=cmd_resource)

    def _test_update_listener(self, args, expected_values):
        resource = 'listener'
        cmd_resource = 'lbaas_listener'
        my_id = 'myid'
        args.insert(0, my_id)
        cmd = listener.UpdateListener(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, my_id,
                                   args, expected_values,
                                   cmd_resource=cmd_resource)

    def test_update_listener(self):
        # lbaas-listener-update myid --name newname.
        self._test_update_listener(['--name', 'newname'],
                                   {'name': 'newname', })
        # lbaas-listener-update myid --description check.
        self._test_update_listener(['--description', 'check'],
                                   {'description': 'check', })
        # lbaas-listener-update myid --connection-limit -1
        self._test_update_listener(['--connection-limit', '-1'],
                                   {'connection_limit': -1, })
        # lbaas-listener-update myid --admin-state-up False.
        self._test_update_listener(['--admin-state-up', 'False'],
                                   {'admin_state_up': 'False', })

    def test_delete_listener(self):
        # lbaas-listener-delete my-id.
        resource = 'listener'
        cmd_resource = 'lbaas_listener'
        cmd = listener.DeleteListener(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args,
                                   cmd_resource=cmd_resource)



# Copyright 2016-2017 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import copy
import logging

from cliff import columns as cliff_columns
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _
from neutronclient.common import utils as nc_utils
from neutronclient.osc import utils as osc_utils
from neutronclient.osc.v2.fwaas import constants as const


LOG = logging.getLogger(__name__)


_attr_map = (
    ('id', 'ID', column_util.LIST_BOTH),
    ('name', 'Name', column_util.LIST_BOTH),
    ('enabled', 'Enabled', column_util.LIST_BOTH),
    ('summary', 'Summary', column_util.LIST_SHORT_ONLY),
    ('description', 'Description', column_util.LIST_LONG_ONLY),
    ('ip_version', 'IP Version', column_util.LIST_LONG_ONLY),
    ('action', 'Action', column_util.LIST_LONG_ONLY),
    ('protocol', 'Protocol', column_util.LIST_LONG_ONLY),
    ('source_ip_address', 'Source IP Address', column_util.LIST_LONG_ONLY),
    ('source_port', 'Source Port', column_util.LIST_LONG_ONLY),
    ('destination_ip_address', 'Destination IP Address',
        column_util.LIST_LONG_ONLY),
    ('destination_port', 'Destination Port', column_util.LIST_LONG_ONLY),
    ('shared', 'Shared', column_util.LIST_LONG_ONLY),
    ('tenant_id', 'Project', column_util.LIST_LONG_ONLY),
    ('source_firewall_group_id', 'Source Firewall Group ID',
     column_util.LIST_LONG_ONLY),
    ('destination_firewall_group_id', 'Destination Firewall Group ID',
     column_util.LIST_LONG_ONLY),
)


def _get_common_parser(parser):
    parser.add_argument(
        '--name',
        metavar='<name>',
        help=_('Name of the firewall rule'))
    parser.add_argument(
        '--description',
        metavar='<description>',
        help=_('Description of the firewall rule'))
    parser.add_argument(
        '--protocol',
        choices=['tcp', 'udp', 'icmp', 'any'],
        type=nc_utils.convert_to_lowercase,
        help=_('Protocol for the firewall rule'))
    parser.add_argument(
        '--action',
        choices=['allow', 'deny', 'reject'],
        type=nc_utils.convert_to_lowercase,
        help=_('Action for the firewall rule'))
    parser.add_argument(
        '--ip-version',
        metavar='<ip-version>',
        choices=['4', '6'],
        help=_('Set IP version 4 or 6 (default is 4)'))
    src_ip_group = parser.add_mutually_exclusive_group()
    src_ip_group.add_argument(
        '--source-ip-address',
        metavar='<source-ip-address>',
        help=_('Source IP address or subnet'))
    src_ip_group.add_argument(
        '--no-source-ip-address',
        action='store_true',
        help=_('Detach source IP address'))
    dst_ip_group = parser.add_mutually_exclusive_group()
    dst_ip_group.add_argument(
        '--destination-ip-address',
        metavar='<destination-ip-address>',
        help=_('Destination IP address or subnet'))
    dst_ip_group.add_argument(
        '--no-destination-ip-address',
        action='store_true',
        help=_('Detach destination IP address'))
    src_port_group = parser.add_mutually_exclusive_group()
    src_port_group.add_argument(
        '--source-port',
        metavar='<source-port>',
        help=_('Source port number or range'
               '(integer in [1, 65535] or range like 123:456)'))
    src_port_group.add_argument(
        '--no-source-port',
        action='store_true',
        help=_('Detach source port number or range'))
    dst_port_group = parser.add_mutually_exclusive_group()
    dst_port_group.add_argument(
        '--destination-port',
        metavar='<destination-port>',
        help=_('Destination port number or range'
               '(integer in [1, 65535] or range like 123:456)'))
    dst_port_group.add_argument(
        '--no-destination-port',
        action='store_true',
        help=_('Detach destination port number or range'))
    shared_group = parser.add_mutually_exclusive_group()
    shared_group.add_argument(
        '--share',
        action='store_true',
        help=_('Share the firewall rule to be used in all projects '
               '(by default, it is restricted to be used by the '
               'current project).'))
    shared_group.add_argument(
        '--no-share',
        action='store_true',
        help=_('Restrict use of the firewall rule to the current project'))
    enable_group = parser.add_mutually_exclusive_group()
    enable_group.add_argument(
        '--enable-rule',
        action='store_true',
        help=_('Enable this rule (default is enabled)'))
    enable_group.add_argument(
        '--disable-rule',
        action='store_true',
        help=_('Disable this rule'))
    src_fwg_group = parser.add_mutually_exclusive_group()
    src_fwg_group.add_argument(
        '--source-firewall-group',
        metavar='<source-firewall-group>',
        help=_('Source firewall group (name or ID)'))
    src_fwg_group.add_argument(
        '--no-source-firewall-group',
        action='store_true',
        help=_('No associated destination firewall group'))
    dst_fwg_group = parser.add_mutually_exclusive_group()
    dst_fwg_group.add_argument(
        '--destination-firewall-group',
        metavar='<destination-firewall-group>',
        help=_('Destination firewall group (name or ID)'))
    dst_fwg_group.add_argument(
        '--no-destination-firewall-group',
        action='store_true',
        help=_('No associated destination firewall group'))
    return parser


def _get_common_attrs(client_manager, parsed_args, is_create=True):
    attrs = {}
    client = client_manager.neutronclient
    if is_create:
        if 'project' in parsed_args and parsed_args.project is not None:
            attrs['tenant_id'] = osc_utils.find_project(
                client_manager.identity,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
    if parsed_args.name:
        attrs['name'] = str(parsed_args.name)
    if parsed_args.description:
        attrs['description'] = str(parsed_args.description)
    if parsed_args.protocol:
        protocol = parsed_args.protocol
        attrs['protocol'] = None if protocol == 'any' else protocol
    if parsed_args.action:
        attrs['action'] = parsed_args.action
    if parsed_args.ip_version:
        attrs['ip_version'] = str(parsed_args.ip_version)
    if parsed_args.source_port:
        attrs['source_port'] = parsed_args.source_port
    if parsed_args.no_source_port:
        attrs['source_port'] = None
    if parsed_args.source_ip_address:
        attrs['source_ip_address'] = parsed_args.source_ip_address
    if parsed_args.no_source_ip_address:
        attrs['source_ip_address'] = None
    if parsed_args.destination_port:
        attrs['destination_port'] = str(parsed_args.destination_port)
    if parsed_args.no_destination_port:
        attrs['destination_port'] = None
    if parsed_args.destination_ip_address:
        attrs['destination_ip_address'] = str(
            parsed_args.destination_ip_address)
    if parsed_args.no_destination_ip_address:
        attrs['destination_ip_address'] = None
    if parsed_args.enable_rule:
        attrs['enabled'] = True
    if parsed_args.disable_rule:
        attrs['enabled'] = False
    if parsed_args.share:
        attrs['shared'] = True
    if parsed_args.no_share:
        attrs['shared'] = False
    if parsed_args.source_firewall_group:
        attrs['source_firewall_group_id'] = client.find_resource(
            const.FWG, parsed_args.source_firewall_group,
            cmd_resource=const.CMD_FWG)['id']
    if parsed_args.no_source_firewall_group:
        attrs['source_firewall_group_id'] = None
    if parsed_args.destination_firewall_group:
        attrs['destination_firewall_group_id'] = client.find_resource(
            const.FWG, parsed_args.destination_firewall_group,
            cmd_resource=const.CMD_FWG)['id']
    if parsed_args.no_destination_firewall_group:
        attrs['destination_firewall_group_id'] = None
    return attrs


class ProtocolColumn(cliff_columns.FormattableColumn):
    def human_readable(self):
        return self._value if self._value else 'any'


_formatters = {'protocol': ProtocolColumn}


class CreateFirewallRule(command.ShowOne):
    _description = _("Create a new firewall rule")

    def get_parser(self, prog_name):
        parser = super(CreateFirewallRule, self).get_parser(prog_name)
        _get_common_parser(parser)
        osc_utils.add_project_owner_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_common_attrs(self.app.client_manager, parsed_args)
        obj = client.create_fwaas_firewall_rule(
            {const.FWR: attrs})[const.FWR]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = utils.get_dict_properties(obj, columns, formatters=_formatters)
        return display_columns, data


class DeleteFirewallRule(command.Command):
    _description = _("Delete firewall rule(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteFirewallRule, self).get_parser(prog_name)
        parser.add_argument(
            const.FWR,
            metavar='<firewall-rule>',
            nargs='+',
            help=_('Firewall rule(s) to delete (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        result = 0
        for fwr in parsed_args.firewall_rule:
            try:
                fwr_id = client.find_resource(
                    const.FWR, fwr, cmd_resource=const.CMD_FWR)['id']
                client.delete_fwaas_firewall_rule(fwr_id)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete Firewall rule with "
                            "name or ID '%(firewall_rule)s': %(e)s"),
                          {const.FWR: fwr, 'e': e})

        if result > 0:
            total = len(parsed_args.firewall_rule)
            msg = (_("%(result)s of %(total)s firewall rule(s) failed "
                     "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class ListFirewallRule(command.Lister):
    _description = _("List firewall rules that belong to a given tenant")

    def get_parser(self, prog_name):
        parser = super(ListFirewallRule, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help=_("List additional fields in output")
        )
        return parser

    def extend_list(self, data, parsed_args):
        ext_data = copy.deepcopy(data)
        for d in ext_data:
            protocol = d['protocol'].upper() if d['protocol'] else 'ANY'
            src_ip = 'none specified'
            dst_ip = 'none specified'
            src_port = '(none specified)'
            dst_port = '(none specified)'
            if 'source_ip_address' in d and d['source_ip_address']:
                src_ip = str(d['source_ip_address']).lower()
            if 'source_port' in d and d['source_port']:
                src_port = '(' + str(d['source_port']).lower() + ')'
            if 'destination_ip_address' in d and d['destination_ip_address']:
                dst_ip = str(d['destination_ip_address']).lower()
            if 'destination_port' in d and d['destination_port']:
                dst_port = '(' + str(d['destination_port']).lower() + ')'
            action = d['action'] if d.get('action') else 'no-action'
            src = 'source(port): ' + src_ip + src_port
            dst = 'dest(port): ' + dst_ip + dst_port
            d['summary'] = ',\n '.join([protocol, src, dst, action])
        return ext_data

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        obj = client.list_fwaas_firewall_rules()[const.FWRS]
        obj_extend = self.extend_list(obj, parsed_args)
        headers, columns = column_util.get_column_definitions(
            _attr_map, long_listing=parsed_args.long)
        return (headers, (utils.get_dict_properties(
            s, columns, formatters=_formatters) for s in obj_extend))


class SetFirewallRule(command.Command):
    _description = _("Set firewall rule properties")

    def get_parser(self, prog_name):
        parser = super(SetFirewallRule, self).get_parser(prog_name)
        _get_common_parser(parser)
        parser.add_argument(
            const.FWR,
            metavar='<firewall-rule>',
            help=_('Firewall rule to set (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_common_attrs(self.app.client_manager,
                                  parsed_args, is_create=False)
        fwr_id = client.find_resource(
            const.FWR, parsed_args.firewall_rule,
            cmd_resource=const.CMD_FWR)['id']
        try:
            client.update_fwaas_firewall_rule(fwr_id, {const.FWR: attrs})
        except Exception as e:
            msg = (_("Failed to set firewall rule '%(rule)s': %(e)s")
                   % {'rule': parsed_args.firewall_rule, 'e': e})
            raise exceptions.CommandError(msg)


class ShowFirewallRule(command.ShowOne):
    _description = _("Display firewall rule details")

    def get_parser(self, prog_name):
        parser = super(ShowFirewallRule, self).get_parser(prog_name)
        parser.add_argument(
            const.FWR,
            metavar='<firewall-rule>',
            help=_('Firewall rule to display (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        fwr_id = client.find_resource(
            const.FWR, parsed_args.firewall_rule,
            cmd_resource=const.CMD_FWR)['id']
        obj = client.show_fwaas_firewall_rule(fwr_id)[const.FWR]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = utils.get_dict_properties(obj, columns, formatters=_formatters)
        return (display_columns, data)


class UnsetFirewallRule(command.Command):
    _description = _("Unset firewall rule properties")

    def get_parser(self, prog_name):
        parser = super(UnsetFirewallRule, self).get_parser(prog_name)
        parser.add_argument(
            const.FWR,
            metavar='<firewall-rule>',
            help=_('Firewall rule to unset (name or ID)'))
        parser.add_argument(
            '--source-ip-address',
            action='store_true',
            help=_('Source IP address or subnet'))
        parser.add_argument(
            '--destination-ip-address',
            action='store_true',
            help=_('Destination IP address or subnet'))
        parser.add_argument(
            '--source-port',
            action='store_true',
            help=_('Source port number or range'
                   '(integer in [1, 65535] or range like 123:456)'))
        parser.add_argument(
            '--destination-port',
            action='store_true',
            help=_('Destination port number or range'
                   '(integer in [1, 65535] or range like 123:456)'))
        parser.add_argument(
            '--share',
            action='store_true',
            help=_('Restrict use of the firewall rule to the current project'))
        parser.add_argument(
            '--enable-rule',
            action='store_true',
            help=_('Disable this rule'))

        parser.add_argument(
            '--source-firewall-group',
            action='store_true',
            help=_('Source firewall group (name or ID)'))

        parser.add_argument(
            '--destination-firewall-group',
            action='store_true',
            help=_('Destination firewall group (name or ID)'))
        return parser

    def _get_attrs(self, client_manager, parsed_args):
        attrs = {}
        if parsed_args.source_ip_address:
            attrs['source_ip_address'] = None
        if parsed_args.source_port:
            attrs['source_port'] = None
        if parsed_args.destination_ip_address:
            attrs['destination_ip_address'] = None
        if parsed_args.destination_port:
            attrs['destination_port'] = None
        if parsed_args.share:
            attrs['shared'] = False
        if parsed_args.enable_rule:
            attrs['enabled'] = False
        if parsed_args.source_firewall_group:
            attrs['source_firewall_group_id'] = None
        if parsed_args.source_firewall_group:
            attrs['destination_firewall_group_id'] = None
        return attrs

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = self._get_attrs(self.app.client_manager, parsed_args)
        fwr_id = client.find_resource(
            const.FWR, parsed_args.firewall_rule,
            cmd_resource=const.CMD_FWR)['id']
        try:
            client.update_fwaas_firewall_rule(fwr_id, {const.FWR: attrs})
        except Exception as e:
            msg = (_("Failed to unset firewall rule '%(rule)s': %(e)s")
                   % {'rule': parsed_args.firewall_rule, 'e': e})
            raise exceptions.CommandError(msg)



# Copyright (c) 2017 Huawei Technologies India Pvt.Limited.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutronclient.osc.v2.sfc import sfc_port_pair
from neutronclient.tests.unit.osc.v2.sfc import fakes


def _get_id(client, id_or_name, resource):
    return id_or_name


class TestCreateSfcPortPair(fakes.TestNeutronClientOSCV2):
    # The new port_pair created
    _port_pair = fakes.FakeSfcPortPair.create_port_pair()

    columns = ('Description',
               'Egress Logical Port',
               'ID',
               'Ingress Logical Port',
               'Name',
               'Project',
               'Service Function Parameters')

    def get_data(self):
        return (
            self._port_pair['description'],
            self._port_pair['egress'],
            self._port_pair['id'],
            self._port_pair['ingress'],
            self._port_pair['name'],
            self._port_pair['project_id'],
            self._port_pair['service_function_parameters']
        )

    def setUp(self):
        super(TestCreateSfcPortPair, self).setUp()
        mock.patch('neutronclient.osc.v2.sfc.sfc_port_pair._get_id',
                   new=_get_id).start()
        self.neutronclient.create_sfc_port_pair = mock.Mock(
            return_value={'port_pair': self._port_pair})
        self.data = self.get_data()

        # Get the command object to test
        self.cmd = sfc_port_pair.CreateSfcPortPair(self.app, self.namespace)

    def test_create_port_pair_default_options(self):
        arglist = [
            "--ingress", self._port_pair['ingress'],
            "--egress", self._port_pair['egress'],
            self._port_pair['name'],
        ]
        verifylist = [
            ('ingress', self._port_pair['ingress']),
            ('egress', self._port_pair['egress']),
            ('name', self._port_pair['name'])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.neutronclient.create_sfc_port_pair.assert_called_once_with({
            'port_pair': {'name': self._port_pair['name'],
                          'ingress': self._port_pair['ingress'],
                          'egress': self._port_pair['egress'],
                          }
        })
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def _test_create_port_pair_all_options(self, correlation):
        arglist = [
            "--description", self._port_pair['description'],
            "--egress", self._port_pair['egress'],
            "--ingress", self._port_pair['ingress'],
            self._port_pair['name'],
            "--service-function-parameters",
            'correlation=%s,weight=1' % correlation,
        ]

        verifylist = [
            ('ingress', self._port_pair['ingress']),
            ('egress', self._port_pair['egress']),
            ('name', self._port_pair['name']),
            ('description', self._port_pair['description']),
            ('service_function_parameters',
             [{'correlation': correlation, 'weight': '1'}])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        if correlation == "None":
            correlation_param = None
        else:
            correlation_param = correlation
        self.neutronclient.create_sfc_port_pair.assert_called_once_with({
            'port_pair': {'name': self._port_pair['name'],
                          'ingress': self._port_pair['ingress'],
                          'egress': self._port_pair['egress'],
                          'description': self._port_pair['description'],
                          'service_function_parameters':
                              {'correlation': correlation_param, 'weight':
                                  '1'},
                          }
        })
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_port_pair_all_options(self):
        self._test_create_port_pair_all_options('None')

    def test_create_port_pair_all_options_mpls(self):
        self._test_create_port_pair_all_options('mpls')


class TestDeleteSfcPortPair(fakes.TestNeutronClientOSCV2):

    _port_pair = fakes.FakeSfcPortPair.create_port_pairs(count=1)

    def setUp(self):
        super(TestDeleteSfcPortPair, self).setUp()
        mock.patch('neutronclient.osc.v2.sfc.sfc_port_pair._get_id',
                   new=_get_id).start()
        self.neutronclient.delete_sfc_port_pair = mock.Mock(return_value=None)
        self.cmd = sfc_port_pair.DeleteSfcPortPair(self.app, self.namespace)

    def test_delete_port_pair(self):
        client = self.app.client_manager.neutronclient
        mock_port_pair_delete = client.delete_sfc_port_pair
        arglist = [
            self._port_pair[0]['id'],
        ]
        verifylist = [
            ('port_pair', self._port_pair[0]['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        mock_port_pair_delete.assert_called_once_with(
            self._port_pair[0]['id'])
        self.assertIsNone(result)


class TestListSfcPortPair(fakes.TestNeutronClientOSCV2):
    _port_pairs = fakes.FakeSfcPortPair.create_port_pairs()
    columns = ('ID', 'Name', 'Ingress Logical Port', 'Egress Logical Port')
    columns_long = ('ID', 'Name', 'Ingress Logical Port',
                    'Egress Logical Port', 'Service Function Parameters',
                    'Description', 'Project')
    _port_pair = _port_pairs[0]
    data = [
        _port_pair['id'],
        _port_pair['name'],
        _port_pair['ingress'],
        _port_pair['egress']
    ]
    data_long = [
        _port_pair['id'],
        _port_pair['name'],
        _port_pair['ingress'],
        _port_pair['egress'],
        _port_pair['service_function_parameters'],
        _port_pair['description']
    ]
    _port_pair1 = {'port_pairs': _port_pair}
    _port_pair_id = _port_pair['id'],

    def setUp(self):
        super(TestListSfcPortPair, self).setUp()
        mock.patch('neutronclient.osc.v2.sfc.sfc_port_pair._get_id',
                   new=_get_id).start()
        self.neutronclient.list_sfc_port_pairs = mock.Mock(
            return_value={'port_pairs': self._port_pairs}
        )
        # Get the command object to test
        self.cmd = sfc_port_pair.ListSfcPortPair(self.app, self.namespace)

    def test_list_port_pairs(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns = self.cmd.take_action(parsed_args)[0]
        port_pairs = self.neutronclient.list_sfc_port_pairs()['port_pairs']
        port_pair = port_pairs[0]
        data = [
            port_pair['id'],
            port_pair['name'],
            port_pair['ingress'],
            port_pair['egress']
        ]
        self.assertEqual(list(self.columns), columns)
        self.assertEqual(self.data, data)

    def test_list_with_long_option(self):
        arglist = ['--long']
        verifylist = [('long', True)]
        port_pairs = self.neutronclient.list_sfc_port_pairs()['port_pairs']
        port_pair = port_pairs[0]
        data = [
            port_pair['id'],
            port_pair['name'],
            port_pair['ingress'],
            port_pair['egress'],
            port_pair['service_function_parameters'],
            port_pair['description']
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns_long = self.cmd.take_action(parsed_args)[0]
        self.assertEqual(list(self.columns_long), columns_long)
        self.assertEqual(self.data_long, data)


class TestSetSfcPortPair(fakes.TestNeutronClientOSCV2):
    _port_pair = fakes.FakeSfcPortPair.create_port_pair()
    _port_pair_name = _port_pair['name']
    _port_pair_id = _port_pair['id']

    def setUp(self):
        super(TestSetSfcPortPair, self).setUp()
        mock.patch('neutronclient.osc.v2.sfc.sfc_port_pair._get_id',
                   new=_get_id).start()
        self.neutronclient.update_sfc_port_pair = mock.Mock(return_value=None)
        self.cmd = sfc_port_pair.SetSfcPortPair(self.app, self.namespace)

    def test_set_port_pair(self):
        client = self.app.client_manager.neutronclient
        mock_port_pair_update = client.update_sfc_port_pair
        arglist = [
            self._port_pair_name,
            '--name', 'name_updated',
            '--description', 'desc_updated'
        ]
        verifylist = [
            ('port_pair', self._port_pair_name),
            ('name', 'name_updated'),
            ('description', 'desc_updated'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        attrs = {'port_pair': {
            'name': 'name_updated',
            'description': 'desc_updated'}
        }
        mock_port_pair_update.assert_called_once_with(self._port_pair_name,
                                                      attrs)
        self.assertIsNone(result)


class TestShowSfcPortPair(fakes.TestNeutronClientOSCV2):

    _pp = fakes.FakeSfcPortPair.create_port_pair()

    data = (
        _pp['description'],
        _pp['egress'],
        _pp['id'],
        _pp['ingress'],
        _pp['name'],
        _pp['project_id'],
        _pp['service_function_parameters'],
    )
    _port_pair = {'port_pair': _pp}
    _port_pair_id = _pp['id']
    columns = (
        'Description',
        'Egress Logical Port',
        'ID',
        'Ingress Logical Port',
        'Name',
        'Project',
        'Service Function Parameters'
    )

    def setUp(self):
        super(TestShowSfcPortPair, self).setUp()
        mock.patch('neutronclient.osc.v2.sfc.sfc_port_pair._get_id',
                   new=_get_id).start()

        self.neutronclient.show_sfc_port_pair = mock.Mock(
            return_value=self._port_pair
        )

        # Get the command object to test
        self.cmd = sfc_port_pair.ShowSfcPortPair(self.app, self.namespace)

    def test_show_port_pair(self):
        client = self.app.client_manager.neutronclient
        mock_port_pair_show = client.show_sfc_port_pair
        arglist = [
            self._port_pair_id,
        ]
        verifylist = [
            ('port_pair', self._port_pair_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        mock_port_pair_show.assert_called_once_with(self._port_pair_id)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import logging
import os

import debtcollector.renames
from keystoneauth1 import access
from keystoneauth1 import adapter
from oslo_serialization import jsonutils
from oslo_utils import importutils
import requests

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.common import utils

osprofiler_web = importutils.try_import("osprofiler.web")

_logger = logging.getLogger(__name__)

if os.environ.get('NEUTRONCLIENT_DEBUG'):
    ch = logging.StreamHandler()
    _logger.setLevel(logging.DEBUG)
    _logger.addHandler(ch)
    _requests_log_level = logging.DEBUG
else:
    _requests_log_level = logging.WARNING

logging.getLogger("requests").setLevel(_requests_log_level)
MAX_URI_LEN = 8192
USER_AGENT = 'python-neutronclient'
REQ_ID_HEADER = 'X-OpenStack-Request-ID'


class HTTPClient(object):
    """Handles the REST calls and responses, include authn."""

    CONTENT_TYPE = 'application/json'

    @debtcollector.renames.renamed_kwarg(
        'tenant_id', 'project_id', replace=True)
    @debtcollector.renames.renamed_kwarg(
        'tenant_name', 'project_name', replace=True)
    def __init__(self, username=None, user_id=None,
                 project_name=None, project_id=None,
                 password=None, auth_url=None,
                 token=None, region_name=None, timeout=None,
                 endpoint_url=None, insecure=False,
                 endpoint_type='publicURL',
                 auth_strategy='keystone', ca_cert=None, log_credentials=False,
                 service_type='network', global_request_id=None,
                 **kwargs):

        self.username = username
        self.user_id = user_id
        self.project_name = project_name
        self.project_id = project_id
        self.password = password
        self.auth_url = auth_url.rstrip('/') if auth_url else None
        self.service_type = service_type
        self.endpoint_type = endpoint_type
        self.region_name = region_name
        self.timeout = timeout
        self.auth_token = token
        self.auth_tenant_id = None
        self.auth_user_id = None
        self.endpoint_url = endpoint_url
        self.auth_strategy = auth_strategy
        self.log_credentials = log_credentials
        self.global_request_id = global_request_id
        if insecure:
            self.verify_cert = False
        else:
            self.verify_cert = ca_cert if ca_cert else True

    def _cs_request(self, *args, **kwargs):
        kargs = {}
        kargs.setdefault('headers', kwargs.get('headers', {}))
        kargs['headers']['User-Agent'] = USER_AGENT

        if 'body' in kwargs:
            kargs['body'] = kwargs['body']

        if self.log_credentials:
            log_kargs = kargs
        else:
            log_kargs = self._strip_credentials(kargs)

        utils.http_log_req(_logger, args, log_kargs)
        try:
            resp, body = self.request(*args, **kargs)
        except requests.exceptions.SSLError as e:
            raise exceptions.SslCertificateValidationError(reason=str(e))
        except Exception as e:
            # Wrap the low-level connection error (socket timeout, redirect
            # limit, decompression error, etc) into our custom high-level
            # connection exception (it is excepted in the upper layers of code)
            _logger.debug("throwing ConnectionFailed : %s", e)
            raise exceptions.ConnectionFailed(reason=str(e))
        utils.http_log_resp(_logger, resp, body)

        # log request-id for each api call
        request_id = resp.headers.get('x-openstack-request-id')
        if request_id:
            _logger.debug('%(method)s call to neutron for '
                          '%(url)s used request id '
                          '%(response_request_id)s',
                          {'method': resp.request.method,
                           'url': resp.url,
                           'response_request_id': request_id})

        if resp.status_code == 401:
            raise exceptions.Unauthorized(message=body)
        return resp, body

    def _strip_credentials(self, kwargs):
        if kwargs.get('body') and self.password:
            log_kwargs = kwargs.copy()
            log_kwargs['body'] = kwargs['body'].replace(self.password,
                                                        'REDACTED')
            return log_kwargs
        else:
            return kwargs

    def authenticate_and_fetch_endpoint_url(self):
        if not self.auth_token:
            self.authenticate()
        elif not self.endpoint_url:
            self.endpoint_url = self._get_endpoint_url()

    def request(self, url, method, body=None, headers=None, **kwargs):
        """Request without authentication."""

        content_type = kwargs.pop('content_type', None) or 'application/json'
        headers = headers or {}
        headers.setdefault('Accept', content_type)

        if body:
            headers.setdefault('Content-Type', content_type)

        if self.global_request_id:
            headers.setdefault(REQ_ID_HEADER, self.global_request_id)

        headers['User-Agent'] = USER_AGENT
        # NOTE(dbelova): osprofiler_web.get_trace_id_headers does not add any
        # headers in case if osprofiler is not initialized.
        if osprofiler_web:
            headers.update(osprofiler_web.get_trace_id_headers())

        resp = requests.request(
            method,
            url,
            data=body,
            headers=headers,
            verify=self.verify_cert,
            timeout=self.timeout,
            **kwargs)

        return resp, resp.text

    def _check_uri_length(self, action):
        uri_len = len(self.endpoint_url) + len(action)
        if uri_len > MAX_URI_LEN:
            raise exceptions.RequestURITooLong(
                excess=uri_len - MAX_URI_LEN)

    def do_request(self, url, method, **kwargs):
        # Ensure client always has correct uri - do not guesstimate anything
        self.authenticate_and_fetch_endpoint_url()
        self._check_uri_length(url)

        # Perform the request once. If we get a 401 back then it
        # might be because the auth token expired, so try to
        # re-authenticate and try again. If it still fails, bail.
        try:
            kwargs['headers'] = kwargs.get('headers') or {}
            if self.auth_token is None:
                self.auth_token = ""
            kwargs['headers']['X-Auth-Token'] = self.auth_token
            resp, body = self._cs_request(self.endpoint_url + url, method,
                                          **kwargs)
            return resp, body
        except exceptions.Unauthorized:
            self.authenticate()
            kwargs['headers'] = kwargs.get('headers') or {}
            kwargs['headers']['X-Auth-Token'] = self.auth_token
            resp, body = self._cs_request(
                self.endpoint_url + url, method, **kwargs)
            return resp, body

    def _extract_service_catalog(self, body):
        """Set the client's service catalog from the response data."""
        self.auth_ref = access.create(body=body)
        self.service_catalog = self.auth_ref.service_catalog
        self.auth_token = self.auth_ref.auth_token
        self.auth_tenant_id = self.auth_ref.tenant_id
        self.auth_user_id = self.auth_ref.user_id

        if not self.endpoint_url:
            self.endpoint_url = self.service_catalog.url_for(
                region_name=self.region_name,
                service_type=self.service_type,
                interface=self.endpoint_type)

    def _authenticate_keystone(self):
        if self.user_id:
            creds = {'userId': self.user_id,
                     'password': self.password}
        else:
            creds = {'username': self.username,
                     'password': self.password}

        if self.project_id:
            body = {'auth': {'passwordCredentials': creds,
                             'tenantId': self.project_id, }, }
        else:
            body = {'auth': {'passwordCredentials': creds,
                             'tenantName': self.project_name, }, }

        if self.auth_url is None:
            raise exceptions.NoAuthURLProvided()

        token_url = self.auth_url + "/tokens"
        resp, resp_body = self._cs_request(token_url, "POST",
                                           body=jsonutils.dumps(body),
                                           content_type="application/json",
                                           allow_redirects=True)
        if resp.status_code != 200:
            raise exceptions.Unauthorized(message=resp_body)
        if resp_body:
            try:
                resp_body = jsonutils.loads(resp_body)
            except ValueError:
                pass
        else:
            resp_body = None
        self._extract_service_catalog(resp_body)

    def _authenticate_noauth(self):
        if not self.endpoint_url:
            message = _('For "noauth" authentication strategy, the endpoint '
                        'must be specified either in the constructor or '
                        'using --os-url')
            raise exceptions.Unauthorized(message=message)

    def authenticate(self):
        if self.auth_strategy == 'keystone':
            self._authenticate_keystone()
        elif self.auth_strategy == 'noauth':
            self._authenticate_noauth()
        else:
            err_msg = _('Unknown auth strategy: %s') % self.auth_strategy
            raise exceptions.Unauthorized(message=err_msg)

    def _get_endpoint_url(self):
        if self.auth_url is None:
            raise exceptions.NoAuthURLProvided()

        url = self.auth_url + '/tokens/%s/endpoints' % self.auth_token
        try:
            resp, body = self._cs_request(url, "GET")
        except exceptions.Unauthorized:
            # rollback to authenticate() to handle case when neutron client
            # is initialized just before the token is expired
            self.authenticate()
            return self.endpoint_url

        body = jsonutils.loads(body)
        for endpoint in body.get('endpoints', []):
            if (endpoint['type'] == 'network' and
                    endpoint.get('region') == self.region_name):
                if self.endpoint_type not in endpoint:
                    raise exceptions.EndpointTypeNotFound(
                        type_=self.endpoint_type)
                return endpoint[self.endpoint_type]

        raise exceptions.EndpointNotFound()

    def get_auth_info(self):
        return {'auth_token': self.auth_token,
                'auth_tenant_id': self.auth_tenant_id,
                'auth_user_id': self.auth_user_id,
                'endpoint_url': self.endpoint_url}

    def get_auth_ref(self):
        return getattr(self, 'auth_ref', None)


class SessionClient(adapter.Adapter):

    def request(self, *args, **kwargs):
        kwargs.setdefault('authenticated', False)
        kwargs.setdefault('raise_exc', False)

        content_type = kwargs.pop('content_type', None) or 'application/json'

        headers = kwargs.get('headers') or {}
        headers.setdefault('Accept', content_type)

        # NOTE(dbelova): osprofiler_web.get_trace_id_headers does not add any
        # headers in case if osprofiler is not initialized.
        if osprofiler_web:
            headers.update(osprofiler_web.get_trace_id_headers())

        try:
            kwargs.setdefault('data', kwargs.pop('body'))
        except KeyError:
            pass

        if kwargs.get('data'):
            headers.setdefault('Content-Type', content_type)

        kwargs['headers'] = headers
        resp = super(SessionClient, self).request(*args, **kwargs)
        return resp, resp.text

    def _check_uri_length(self, url):
        uri_len = len(self.endpoint_url) + len(url)
        if uri_len > MAX_URI_LEN:
            raise exceptions.RequestURITooLong(
                excess=uri_len - MAX_URI_LEN)

    def do_request(self, url, method, **kwargs):
        kwargs.setdefault('authenticated', True)
        self._check_uri_length(url)
        return self.request(url, method, **kwargs)

    @property
    def endpoint_url(self):
        # NOTE(jamielennox): This is used purely by the CLI and should be
        # removed when the CLI gets smarter.
        return self.get_endpoint()

    @property
    def auth_token(self):
        # NOTE(jamielennox): This is used purely by the CLI and should be
        # removed when the CLI gets smarter.
        return self.get_token()

    def authenticate(self):
        # NOTE(jamielennox): This is used purely by the CLI and should be
        # removed when the CLI gets smarter.
        self.get_token()

    def get_auth_info(self):
        auth_info = {'auth_token': self.auth_token,
                     'endpoint_url': self.endpoint_url}

        # NOTE(jamielennox): This is the best we can do here. It will work
        # with identity plugins which is the primary case but we should
        # deprecate it's usage as much as possible.
        try:
            get_access = (self.auth or self.session.auth).get_access
        except AttributeError:
            pass
        else:
            auth_ref = get_access(self.session)

            auth_info['auth_tenant_id'] = auth_ref.project_id
            auth_info['auth_user_id'] = auth_ref.user_id

        return auth_info

    def get_auth_ref(self):
        return self.session.auth.get_auth_ref(self.session)


# FIXME(bklei): Should refactor this to use kwargs and only
# explicitly list arguments that are not None.
@debtcollector.renames.renamed_kwarg('tenant_id', 'project_id', replace=True)
@debtcollector.renames.renamed_kwarg(
    'tenant_name', 'project_name', replace=True)
def construct_http_client(username=None,
                          user_id=None,
                          project_name=None,
                          project_id=None,
                          password=None,
                          auth_url=None,
                          token=None,
                          region_name=None,
                          timeout=None,
                          endpoint_url=None,
                          insecure=False,
                          endpoint_type='public',
                          log_credentials=None,
                          auth_strategy='keystone',
                          ca_cert=None,
                          service_type='network',
                          session=None,
                          global_request_id=None,
                          **kwargs):

    if session:
        kwargs.setdefault('user_agent', USER_AGENT)
        kwargs.setdefault('interface', endpoint_type)
        return SessionClient(session=session,
                             service_type=service_type,
                             region_name=region_name,
                             global_request_id=global_request_id,
                             **kwargs)
    else:
        # FIXME(bklei): username and password are now optional. Need
        # to test that they were provided in this mode.  Should also
        # refactor to use kwargs.
        return HTTPClient(username=username,
                          password=password,
                          project_id=project_id,
                          project_name=project_name,
                          user_id=user_id,
                          auth_url=auth_url,
                          token=token,
                          endpoint_url=endpoint_url,
                          insecure=insecure,
                          timeout=timeout,
                          region_name=region_name,
                          endpoint_type=endpoint_type,
                          service_type=service_type,
                          ca_cert=ca_cert,
                          log_credentials=log_credentials,
                          auth_strategy=auth_strategy,
                          global_request_id=global_request_id)



# Copyright 2013 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient.neutron import v2_0 as neutronV20


class ListServiceProvider(neutronV20.ListCommand):
    """List service providers."""

    resource = 'service_provider'
    list_columns = ['service_type', 'name', 'default']
    _formatters = {}
    pagination_support = True
    sorting_support = True
